@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24540 $
+ * $Revision: 25387 $
  * $Author: rjongbloed $
- * $Date: 2010-06-28 20:44:22 -0500 (Mon, 28 Jun 2010) $
+ * $Date: 2011-03-22 22:51:09 -0500 (Tue, 22 Mar 2011) $
  */
 
 #ifndef PTLIB_HTTP_H
@@ -65,10 +65,8 @@ class PHTTPSpace : public PContainer
 
   // New functions for class.
     enum AddOptions {
-      /// Generate error if resource already exists
-      ErrorOnExist,
-      /// Overwrite the existing resource at URL location
-      Overwrite
+      ErrorOnExist, ///< Generate error if resource already exists
+      Overwrite     ///< Overwrite the existing resource at URL location
     };
 
 
@@ -81,7 +79,7 @@ class PHTTPSpace : public PContainer
        path to another resource but not if it is a leaf node.
 
        @return
-       PTrue if resource added, PFalse if failed.
+       true if resource added, false if failed.
      */
     PBoolean AddResource(
       PHTTPResource * resource, ///< Resource to add to the name space.
@@ -94,7 +92,7 @@ class PHTTPSpace : public PContainer
        path to another resource then the function will fail.
 
        @return
-       PTrue if resource deleted, PFalse if failed.
+       true if resource deleted, false if failed.
      */
     PBoolean DelResource(
       const PURL & url          ///< URL to search for in the name space.
@@ -148,7 +146,7 @@ class PHTTPSpace : public PContainer
     } * root;
 
   private:
-    PBoolean SetSize(PINDEX) { return PFalse; }
+    PBoolean SetSize(PINDEX) { return false; }
 };
 
 #ifdef TRACE
@@ -205,7 +203,7 @@ class PHTTP : public PInternetProtocol
       Conflict,                    ///< 409 - resource conflict on action
       Gone,                        ///< 410 - resource gone away
       LengthRequired,              ///< 411 - no Content-Length
-      UnlessTrue,                  ///< 412 - no Range header for PTrue Unless
+      UnlessTrue,                  ///< 412 - no Range header for true Unless
       InternalServerError = 500,   ///< 500 - server has encountered an unexpected error
       NotImplemented,              ///< 501 - server does not implement request
       BadGateway,                  ///< 502 - error whilst acting as gateway
@@ -214,34 +212,34 @@ class PHTTP : public PInternetProtocol
     };
 
     // Common MIME header tags
-    static const PString & AllowTag();
-    static const PString & AuthorizationTag();
-    static const PString & ContentEncodingTag();
-    static const PString & ContentLengthTag();
-    static const PString & ContentTypeTag() { return PMIMEInfo::ContentTypeTag(); }
-    static const PString & DateTag();
-    static const PString & ExpiresTag();
-    static const PString & FromTag();
-    static const PString & IfModifiedSinceTag();
-    static const PString & LastModifiedTag();
-    static const PString & LocationTag();
-    static const PString & PragmaTag();
-    static const PString & PragmaNoCacheTag();
-    static const PString & RefererTag();
-    static const PString & ServerTag();
-    static const PString & UserAgentTag();
-    static const PString & WWWAuthenticateTag();
-    static const PString & MIMEVersionTag();
-    static const PString & ConnectionTag();
-    static const PString & KeepAliveTag();
-    static const PString & TransferEncodingTag();
-    static const PString & ChunkedTag();
-    static const PString & ProxyConnectionTag();
-    static const PString & ProxyAuthorizationTag();
-    static const PString & ProxyAuthenticateTag();
-    static const PString & ForwardedTag();
-    static const PString & SetCookieTag();
-    static const PString & CookieTag();
+    static const PCaselessString & AllowTag();
+    static const PCaselessString & AuthorizationTag();
+    static const PCaselessString & ContentEncodingTag();
+    static const PCaselessString & ContentLengthTag();
+    static const PCaselessString & ContentTypeTag() { return PMIMEInfo::ContentTypeTag(); }
+    static const PCaselessString & DateTag();
+    static const PCaselessString & ExpiresTag();
+    static const PCaselessString & FromTag();
+    static const PCaselessString & IfModifiedSinceTag();
+    static const PCaselessString & LastModifiedTag();
+    static const PCaselessString & LocationTag();
+    static const PCaselessString & PragmaTag();
+    static const PCaselessString & PragmaNoCacheTag();
+    static const PCaselessString & RefererTag();
+    static const PCaselessString & ServerTag();
+    static const PCaselessString & UserAgentTag();
+    static const PCaselessString & WWWAuthenticateTag();
+    static const PCaselessString & MIMEVersionTag();
+    static const PCaselessString & ConnectionTag();
+    static const PCaselessString & KeepAliveTag();
+    static const PCaselessString & TransferEncodingTag();
+    static const PCaselessString & ChunkedTag();
+    static const PCaselessString & ProxyConnectionTag();
+    static const PCaselessString & ProxyAuthorizationTag();
+    static const PCaselessString & ProxyAuthenticateTag();
+    static const PCaselessString & ForwardedTag();
+    static const PCaselessString & SetCookieTag();
+    static const PCaselessString & CookieTag();
 
   protected:
     /** Create a TCP/IP HTTP protocol channel.
@@ -253,7 +251,7 @@ class PHTTP : public PInternetProtocol
        <CODE>lastResponseCode</CODE> and <CODE>lastResponseInfo</CODE>.
 
        The default bahaviour looks for a space or a '-' and splits the code
-       and info either side of that character, then returns PFalse.
+       and info either side of that character, then returns false.
 
        @return
        Position of continuation character in response, 0 if no continuation
@@ -433,7 +431,7 @@ class PHTTPClientDigestAuthentication : public PHTTPClientAuthentication
       }
       else
          PError << "HTTP conection failed." << endl;
-      </PRE></CODE>
+      </CODE></PRE>
  */
 class PHTTPClient : public PHTTP
 {
@@ -452,7 +450,7 @@ class PHTTPClient : public PHTTP
        200!
 
        @return
-       PTrue if all of header returned and ready to receive body.
+       true if all of header returned and ready to receive body.
      */
     int ExecuteCommand(
       Commands cmd,
@@ -505,7 +503,7 @@ class PHTTPClient : public PHTTP
         type is acceptable.
 
        @return
-       PTrue if document is being transferred.
+       true if document is being transferred.
      */
     PBoolean GetTextDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
@@ -516,7 +514,7 @@ class PHTTPClient : public PHTTP
     /** Get the document specified by the URL.
 
        @return
-       PTrue if document is being transferred.
+       true if document is being transferred.
      */
     PBoolean GetDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
@@ -527,7 +525,7 @@ class PHTTPClient : public PHTTP
     /** Get the header for the document specified by the URL.
 
        @return
-       PTrue if document header is being transferred.
+       true if document header is being transferred.
      */
     PBoolean GetHeader(
       const PURL & url,         ///< Universal Resource Locator for document.
@@ -539,7 +537,7 @@ class PHTTPClient : public PHTTP
     /** Post the data specified to the URL.
 
        @return
-       PTrue if document is being transferred.
+       true if document is being transferred.
      */
     PBoolean PostData(
       const PURL & url,       ///< Universal Resource Locator for document.
@@ -551,7 +549,7 @@ class PHTTPClient : public PHTTP
     /** Post the data specified to the URL.
 
        @return
-       PTrue if document is being transferred.
+       true if document is being transferred.
      */
     PBoolean PostData(
       const PURL & url,       ///< Universal Resource Locator for document.
@@ -629,7 +627,7 @@ class PHTTPServer;
 
 /** This object describes the connectiono associated with a HyperText Transport
    Protocol request. This information is required by handler functions on
-   #PHTTPResource# descendant classes to manage the connection correctly.
+   <code>PHTTPResource</code> descendant classes to manage the connection correctly.
 */
 class PHTTPConnectionInfo : public PObject
 {
@@ -709,15 +707,24 @@ class PHTTPConnectionInfo : public PObject
 
 /** A TCP/IP socket for the HyperText Transfer Protocol version 1.0.
 
-    When acting as a server, a descendant class would be created to override
-    at least the #HandleOpenMailbox()#, #HandleSendMessage()# and
-    #HandleDeleteMessage()# functions. Other functions may be overridden
-    for further enhancement to the sockets capabilities, but these will give a
-    basic POP3 server functionality.
-
     The server socket thread would continuously call the
-    #ProcessMessage()# function until it returns PFalse. This will then
-    call the appropriate virtual function on parsing the POP3 protocol.
+    ProcessCommand() function until it returns false. This will then
+    call the appropriate virtual function on parsing the HTTP protocol.
+    <PRE><CODE>
+    PTCPSocket socket(80);
+    if (!socket.Listen())
+      return;
+
+    PHTTPSpace httpNameSpace;
+    httpNameSpace.AddResource(new PHTTPDirectory("data", "data"))
+
+    PHTTServer httpServer(httpNameSpace);
+    if (!httpServer.Open(socket))
+      return;
+
+    while (httpServer.ProcessCommand())
+      ;
+    </CODE></PRE>
  */
 class PHTTPServer : public PHTTP
 {
@@ -772,13 +779,13 @@ class PHTTPServer : public PHTTP
     /** Handle a GET command from a client.
 
        The default implementation looks up the URL in the name space declared by
-       the #PHTTPSpace# class tree and despatches to the
-       #PHTTPResource# object contained therein.
+       the <code>PHTTPSpace</code> class tree and despatches to the
+       <code>PHTTPResource</code> object contained therein.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnGET(
       const PURL & url,                    ///< Universal Resource Locator for document.
@@ -791,13 +798,13 @@ class PHTTPServer : public PHTTP
     /** Handle a HEAD command from a client.
 
        The default implemetation looks up the URL in the name space declared by
-       the #PHTTPSpace# class tree and despatches to the
-       #PHTTPResource# object contained therein.
+       the <code>PHTTPSpace</code> class tree and despatches to the
+       <code>PHTTPResource</code> object contained therein.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnHEAD(
       const PURL & url,                   ///< Universal Resource Locator for document.
@@ -808,13 +815,13 @@ class PHTTPServer : public PHTTP
     /** Handle a POST command from a client.
 
        The default implementation looks up the URL in the name space declared by
-       the #PHTTPSpace# class tree and despatches to the
-       #PHTTPResource# object contained therein.
+       the <code>PHTTPSpace</code> class tree and despatches to the
+       <code>PHTTPResource</code> object contained therein.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnPOST(
       const PURL & url,                   ///< Universal Resource Locator for document.
@@ -826,14 +833,14 @@ class PHTTPServer : public PHTTP
     /** Handle a proxy command request from a client. This will only get called
        if the request was not for this particular server. If it was a proxy
        request for this server (host and port number) then the appropriate
-       #OnGET()#, #OnHEAD()# or #OnPOST()# command is called.
+       <code>OnGET()</code>, <code>OnHEAD()</code> or <code>OnPOST()</code> command is called.
 
        The default implementation returns OnError(BadGateway).
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnProxy(
       const PHTTPConnectionInfo & conInfo   ///<  HTTP connection information
@@ -851,7 +858,7 @@ class PHTTPServer : public PHTTP
     /** Handle an unknown command.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
      */
     virtual PBoolean OnUnknown(
       const PCaselessString & command,         ///< Complete command line received.
@@ -874,7 +881,7 @@ class PHTTPServer : public PHTTP
        nothing.
 
        @return
-       PTrue if requires v1.1 chunked transfer encoding.
+       true if requires v1.1 chunked transfer encoding.
      */
     PBoolean StartResponse(
       StatusCode code,      ///< Status code for the response.
@@ -889,7 +896,7 @@ class PHTTPServer : public PHTTP
        viewer.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
      */
     virtual PBoolean OnError(
       StatusCode code,                         ///< Status code for the error response.
@@ -901,7 +908,7 @@ class PHTTPServer : public PHTTP
      */
     void SetDefaultMIMEInfo(
       PMIMEInfo & info,      ///< Extra MIME information in command.
-      const PHTTPConnectionInfo & connectInfo
+      const PHTTPConnectionInfo & connectInfo   ///< Connection information
     );
 
     /**Get the connection info for this connection.
@@ -922,7 +929,7 @@ class PHTTPServer : public PHTTP
 // PHTTPRequest
 
 /** This object describes a HyperText Transport Protocol request. An individual
-   request is passed to handler functions on #PHTTPResource# descendant
+   request is passed to handler functions on <code>PHTTPResource</code> descendant
    classes.
  */
 class PHTTPRequest : public PObject
@@ -979,7 +986,7 @@ class PHTTPAuthority : public PObject
        the realm specified by the class instance.
 
        @return
-       PTrue if the user and password are authorised in the realm.
+       true if the user and password are authorised in the realm.
      */
     virtual PBoolean Validate(
       const PHTTPRequest & request,  ///< Request information.
@@ -990,10 +997,10 @@ class PHTTPAuthority : public PObject
        distinguish between net requiring authorisation and requiring autorisation
        but having no password.
 
-       The default behaviour is to return PTrue.
+       The default behaviour is to return true.
 
        @return
-       PTrue if the authorisation in the realm is to be applied.
+       true if the authorisation in the realm is to be applied.
      */
     virtual PBoolean IsActive() const;
 
@@ -1027,7 +1034,7 @@ class PHTTPSimpleAuth : public PHTTPAuthority
 
   // Overrides from class PObject.
     /** Create a copy of the class on the heap. This is used by the
-       #PHTTPResource# classes for maintaining authorisation to
+       <code>PHTTPResource</code> classes for maintaining authorisation to
        resources.
 
        @return
@@ -1051,7 +1058,7 @@ class PHTTPSimpleAuth : public PHTTPAuthority
        the realm specified by the class instance.
 
        @return
-       PTrue if the user and password are authorised in the realm.
+       true if the user and password are authorised in the realm.
      */
     virtual PBoolean Validate(
       const PHTTPRequest & request,  ///< Request information.
@@ -1062,10 +1069,10 @@ class PHTTPSimpleAuth : public PHTTPAuthority
        distinguish between net requiring authorisation and requiring autorisation
        but having no password.
 
-       The default behaviour is to return PTrue.
+       The default behaviour is to return true.
 
        @return
-       PTrue if the authorisation in the realm is to be applied.
+       true if the authorisation in the realm is to be applied.
      */
     virtual PBoolean IsActive() const;
 
@@ -1114,7 +1121,7 @@ class PHTTPMultiSimpAuth : public PHTTPAuthority
 
   // Overrides from class PObject.
     /** Create a copy of the class on the heap. This is used by the
-       #PHTTPResource# classes for maintaining authorisation to
+       <code>PHTTPResource</code> classes for maintaining authorisation to
        resources.
 
        @return
@@ -1138,7 +1145,7 @@ class PHTTPMultiSimpAuth : public PHTTPAuthority
        the realm specified by the class instance.
 
        @return
-       PTrue if the user and password are authorised in the realm.
+       true if the user and password are authorised in the realm.
      */
     virtual PBoolean Validate(
       const PHTTPRequest & request,  ///< Request information.
@@ -1149,10 +1156,10 @@ class PHTTPMultiSimpAuth : public PHTTPAuthority
        distinguish between net requiring authorisation and requiring autorisation
        but having no password.
 
-       The default behaviour is to return PTrue.
+       The default behaviour is to return true.
 
        @return
-       PTrue if the authorisation in the realm is to be applied.
+       true if the authorisation in the realm is to be applied.
      */
     virtual PBoolean IsActive() const;
 
@@ -1177,7 +1184,7 @@ class PHTTPMultiSimpAuth : public PHTTPAuthority
 // PHTTPResource
 
 /** This object describes a HyperText Transport Protocol resource. A tree of
-   these resources are available to the #PHTTPSocket# class.
+   these resources are available to the <code>PHTTPServer</code> class.
  */
 class PHTTPResource : public PObject
 {
@@ -1256,13 +1263,13 @@ class PHTTPResource : public PObject
     /** Handle the GET command passed from the HTTP socket.
 
        The default action is to check the authorisation for the resource and
-       call the virtuals #LoadHeaders()# and #OnGETData()# to get
+       call the virtuals <code>LoadHeaders()</code> and <code>OnGETData()</code> to get
        a resource to be sent to the socket.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close.
+       true if the connection may persist, false if the connection must close.
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnGET(
       PHTTPServer & server,       ///< HTTP server that received the request
@@ -1273,12 +1280,12 @@ class PHTTPResource : public PObject
 
     /**Send the data associated with a GET command.
 
-       The default action calls #SendData()#.
+       The default action calls <code>SendData()</code>.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close.
+       true if the connection may persist, false if the connection must close.
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
     */
     virtual PBoolean OnGETData(
       PHTTPServer & server,                       ///< HTTP server that received the request
@@ -1290,13 +1297,13 @@ class PHTTPResource : public PObject
     /** Handle the HEAD command passed from the HTTP socket.
 
        The default action is to check the authorisation for the resource and
-       call the virtual #LoadHeaders()# to get the header information to
+       call the virtual <code>LoadHeaders()</code> to get the header information to
        be sent to the socket.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnHEAD(
       PHTTPServer & server,       ///< HTTP server that received the request
@@ -1308,13 +1315,13 @@ class PHTTPResource : public PObject
     /** Handle the POST command passed from the HTTP socket.
 
        The default action is to check the authorisation for the resource and
-       call the virtual #Post()# function to handle the data being
+       call the virtual <code>Post()</code> function to handle the data being
        received.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
      */
     virtual PBoolean OnPOST(
       PHTTPServer & server,         ///< HTTP server that received the request
@@ -1326,12 +1333,12 @@ class PHTTPResource : public PObject
 
     /**Send the data associated with a POST command.
 
-       The default action calls #Post()#.
+       The default action calls <code>Post()</code>.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close.
+       true if the connection may persist, false if the connection must close.
        If there is no ContentLength field in the response, this value must
-       be PFalse for correct operation.
+       be false for correct operation.
     */
     virtual PBoolean OnPOSTData(
       PHTTPRequest & request,        ///< request information
@@ -1342,7 +1349,7 @@ class PHTTPResource : public PObject
        specified.
 
        @return
-       PTrue if has been modified since.
+       true if has been modified since.
      */
     virtual PBoolean IsModifiedSince(
       const PTime & when    ///< Time to see if modified later than
@@ -1376,7 +1383,7 @@ class PHTTPResource : public PObject
        required by the resource and return the status for the load.
 
        @return
-       PTrue if all OK, PFalse if an error occurred.
+       true if all OK, false if an error occurred.
      */
     virtual PBoolean LoadHeaders(
       PHTTPRequest & request    ///<  Information on this request.
@@ -1384,7 +1391,7 @@ class PHTTPResource : public PObject
 
     /**Send the data associated with a command.
 
-       The default action is to call the virtual #LoadData()# to get a
+       The default action is to call the virtual <code>LoadData()</code> to get a
        resource to be sent to the socket.
     */
     virtual void SendData(
@@ -1393,11 +1400,11 @@ class PHTTPResource : public PObject
 
     /** Get a block of data that the resource contains.
 
-       The default behaviour is to call the #LoadText()# function and
-       if successful, call the #OnLoadedText()# function.
+       The default behaviour is to call the <code>LoadText()</code> function and
+       if successful, call the <code>OnLoadedText()</code> function.
 
        @return
-       PTrue if there is still more to load.
+       true if there is still more to load.
      */
     virtual PBoolean LoadData(
       PHTTPRequest & request,    ///<  Information on this request.
@@ -1406,8 +1413,8 @@ class PHTTPResource : public PObject
 
     /** Get a block of text data (eg HTML) that the resource contains.
 
-       The default behaviour is to assert, one of #LoadText()# or
-       #LoadData()# functions must be overridden for correct operation.
+       The default behaviour is to assert, one of <code>LoadText()</code> or
+       <code>LoadData()</code> functions must be overridden for correct operation.
 
        @return
        String for loaded text.
@@ -1433,7 +1440,7 @@ class PHTTPResource : public PObject
        success.
 
        @return
-       PTrue if the connection may persist, PFalse if the connection must close
+       true if the connection may persist, false if the connection must close
      */
     virtual PBoolean Post(
       PHTTPRequest & request,       ///<  Information on this request.
@@ -1463,18 +1470,15 @@ class PHTTPResource : public PObject
       PHTTPServer & server,       ///<  HTTP server that received the request
       const PURL & url,           ///<  Universal Resource Locator for document.
       const PMIMEInfo & info,     ///<  Extra MIME information in command.
-      const PHTTPConnectionInfo & conInfo,
-      PBoolean  IsGet
+      const PHTTPConnectionInfo & conInfo,  ///< Connection information
+      PBoolean isGet              ///< Flag indicating is GET or HEAD
     );
 
-    /// Base URL for the resource, may accept URLS with a longer hierarchy
-    PURL             baseURL;
-    /// MIME content type for the resource
-    PString          contentType;
-    /// Authorisation method for the resource
-    PHTTPAuthority * authority;
-    /// COunt of number of times resource was accessed.
-    volatile DWORD   hitCount;
+
+    PURL             baseURL;     ///< Base URL for the resource, may accept URLS with a longer hierarchy
+    PString          contentType; ///< MIME content type for the resource
+    PHTTPAuthority * authority;   ///< Authorisation method for the resource
+    volatile DWORD   hitCount;    ///< COunt of number of times resource was accessed.
 };
 
 
@@ -1528,7 +1532,7 @@ class PHTTPString : public PHTTPResource
        required by the resource and return the status for the load.
 
        @return
-       PTrue if all OK, PFalse if an error occurred.
+       true if all OK, false if an error occurred.
      */
     virtual PBoolean LoadHeaders(
       PHTTPRequest & request    // Information on this request.
@@ -1536,8 +1540,8 @@ class PHTTPString : public PHTTPResource
 
     /** Get a block of text data (eg HTML) that the resource contains.
 
-       The default behaviour is to assert, one of #LoadText()# or
-       #LoadData()# functions must be overridden for correct operation.
+       The default behaviour is to assert, one of <code>LoadText()</code> or
+       <code>LoadData()</code> functions must be overridden for correct operation.
 
        @return
        String for loaded text.
@@ -1572,7 +1576,7 @@ class PHTTPString : public PHTTPResource
 /** This object describes a HyperText Transport Protocol resource which is a
    single file. The file can be anywhere in the file system and is mapped to
    the specified URL location in the HTTP name space defined by the
-   #PHTTPSpace# class.
+   <code>PHTTPSpace</code> class.
  */
 class PHTTPFile : public PHTTPResource
 {
@@ -1632,7 +1636,7 @@ class PHTTPFile : public PHTTPResource
        required by the resource and return the status for the load.
 
        @return
-       PTrue if all OK, PFalse if an error occurred.
+       true if all OK, false if an error occurred.
      */
     virtual PBoolean LoadHeaders(
       PHTTPRequest & request    // Information on this request.
@@ -1641,7 +1645,7 @@ class PHTTPFile : public PHTTPResource
     /** Get a block of data that the resource contains.
 
        @return
-       PTrue if more to load.
+       true if more to load.
      */
     virtual PBoolean LoadData(
       PHTTPRequest & request,    // Information on this request.
@@ -1650,8 +1654,8 @@ class PHTTPFile : public PHTTPResource
 
     /** Get a block of text data (eg HTML) that the resource contains.
 
-       The default behaviour is to assert, one of #LoadText()# or
-       #LoadData()# functions must be overridden for correct operation.
+       The default behaviour is to assert, one of <code>LoadText()</code> or
+       <code>LoadData()</code> functions must be overridden for correct operation.
 
        @return
        String for loaded text.
@@ -1695,7 +1699,7 @@ class PHTTPFileRequest : public PHTTPRequest
 /** This object describes a HyperText Transport Protocol resource which is a
    single file. The file can be anywhere in the file system and is mapped to
    the specified URL location in the HTTP name space defined by the
-   #PHTTPSpace# class.
+   <code>PHTTPSpace</code> class.
 
    The difference between this and PHTTPFile is that it continually outputs
    the contents of the file, as per the unix "tail -f" command.
@@ -1746,7 +1750,7 @@ class PHTTPTailFile : public PHTTPFile
        required by the resource and return the status for the load.
 
        @return
-       PTrue if all OK, PFalse if an error occurred.
+       true if all OK, false if an error occurred.
      */
     virtual PBoolean LoadHeaders(
       PHTTPRequest & request    // Information on this request.
@@ -1755,7 +1759,7 @@ class PHTTPTailFile : public PHTTPFile
     /** Get a block of data that the resource contains.
 
        @return
-       PTrue if more to load.
+       true if more to load.
      */
     virtual PBoolean LoadData(
       PHTTPRequest & request,    // Information on this request.
@@ -1770,13 +1774,13 @@ class PHTTPTailFile : public PHTTPFile
 /** This object describes a HyperText Transport Protocol resource which is a
    set of files in a directory. The directory can be anywhere in the file
    system and is mapped to the specified URL location in the HTTP name space
-   defined by the #PHTTPSpace# class.
+   defined by the <code>PHTTPSpace</code> class.
 
    All subdirectories and files are available as URL names in the HTTP name
    space. This effectively grafts a file system directory tree onto the URL
    name space tree.
 
-   See the #PMIMEInfo# class for more information on the mappings between
+   See the <code>PMIMEInfo</code> class for more information on the mappings between
    file types and MIME types.
  */
 class PHTTPDirectory : public PHTTPFile
@@ -1785,13 +1789,13 @@ class PHTTPDirectory : public PHTTPFile
 
   public:
     PHTTPDirectory(
-      const PURL & url,            /// Name of the resource in URL space.
-      const PDirectory & dir       /// Location of file in file system.
+      const PURL & url,            ///< Name of the resource in URL space.
+      const PDirectory & dir       ///< Location of file in file system.
     );
     PHTTPDirectory(
-      const PURL & url,            /// Name of the resource in URL space.
-      const PDirectory & dir,      /// Location of file in file system.
-      const PHTTPAuthority & auth  /// Authorisation for the resource.
+      const PURL & url,            ///< Name of the resource in URL space.
+      const PDirectory & dir,      ///< Location of file in file system.
+      const PHTTPAuthority & auth  ///< Authorisation for the resource.
     );
     // Construct a new directory resource for HTTP.
 
@@ -1814,22 +1818,22 @@ class PHTTPDirectory : public PHTTPFile
        required by the resource and return the status for the load.
 
        @return
-       PTrue if all OK, PFalse if an error occurred.
+       true if all OK, false if an error occurred.
      */
     virtual PBoolean LoadHeaders(
-      PHTTPRequest & request    /// Information on this request.
+      PHTTPRequest & request    ///< Information on this request.
     );
 
     /** Get a block of text data (eg HTML) that the resource contains.
 
-       The default behaviour is to assert, one of #LoadText()# or
-       #LoadData()# functions must be overridden for correct operation.
+       The default behaviour is to assert, one of <code>LoadText()</code> or
+       <code>LoadData()</code> functions must be overridden for correct operation.
 
        @return
        String for loaded text.
      */
     virtual PString LoadText(
-      PHTTPRequest & request    /// Information on this request.
+      PHTTPRequest & request    ///< Information on this request.
     );
 
     /** Enable or disable access control using .access files. A directory tree containing
@@ -1844,7 +1848,7 @@ class PHTTPDirectory : public PHTTPFile
 
     /** Enable or disable directory listings when a default directory file does not exist
      */
-    void AllowDirectories(PBoolean enable = PTrue);
+    void AllowDirectories(PBoolean enable = true);
 
   protected:
     PBoolean CheckAuthority(
@@ -1877,6 +1881,10 @@ class PHTTPDirRequest : public PHTTPFileRequest
     PFilePath realPath;
 };
 
+
+PFACTORY_LOAD(PURL_HttpLoader);
+
+
 #endif // P_HTTP
 
 #endif // PTLIB_HTTP_H
