@@ -21,9 +21,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 21788 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
  */
 
 #ifndef PTLIB_VSDL_H
@@ -32,11 +32,7 @@
 #if P_SDL
 
 #include <ptlib.h>
-#if defined(P_FREEBSD)
-#include <SDL11/SDL.h>
-#else
 #include <SDL/SDL.h>
-#endif
 
 #include <ptlib/videoio.h>
 
@@ -66,7 +62,7 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
     */
     virtual PBoolean Open(
       const PString & /*deviceName*/,   ///< Device name to open
-      PBoolean /*startImmediate*/ = PTrue    ///< Immediately start device
+      PBoolean /*startImmediate*/ = true    ///< Immediately start device
     );
   
     /**Synonymous with the destructor.
@@ -78,14 +74,14 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
     virtual PBoolean IsOpen();
   
     /**Set the colour format to be used.
-       Note that this function does not do any conversion. If it returns PTrue
+       Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
 
        To utilise an internal converter use the SetColourFormatConverter()
        function.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat ///< New colour format for device.
@@ -97,7 +93,7 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -119,7 +115,7 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
       unsigned width,
       unsigned height,
       const BYTE * data,
-      PBoolean endFrame = PTrue
+      PBoolean endFrame = true
     );
 
   protected:
@@ -142,8 +138,6 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
 typedef PVideoOutputDevice_SDL PSDLVideoDevice; // Backward compatibility
 
 
-PWLIB_STATIC_LOAD_PLUGIN(SDL, PVideoOutputDevice);
-
 #endif    // P_SDL
 
 #endif // PTLIB_VSDL_H
