@@ -16,9 +16,9 @@
  * Copied by Derek Smithies, 1)Add soundtest code from ohphone.
  *                           2)Add headers.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 22126 $
+ * $Author: dereksmithies $
+ * $Date: 2009-03-01 17:04:53 -0600 (Sun, 01 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -40,12 +40,13 @@ void Audio::Main()
 {
   PArgList & args = GetArguments();
   args.Parse("r.    "
-       "f.    "
+       "f.    "	     
        "h.    "
 #if PTRACING
              "o-output:"             "-no-output."
              "t-trace."              "-no-trace."
 #endif
+       "p:    "
        "v.    "
        "w:    "
        "s:    ");
@@ -54,9 +55,10 @@ void Audio::Main()
     cout << "usage: audio " 
          << endl
          << "     -r        : report available sound devices" << endl
-         << "     -f.       : do a full duplex sound test on a sound device" << endl
+         << "     -f        : do a full duplex sound test on a sound device" << endl
          << "     -s  dev   : use this device in full duplex test " << endl
          << "     -h        : get help on usage " << endl
+         << "     -p file   : play audio from the file out the specified sound device" << endl
          << "     -v        : report program version " << endl
 	 << "     -w file   : write the captured audio to this file" << endl
 #if PTRACING
@@ -85,34 +87,26 @@ void Audio::Main()
   }
   
 
-  cout << "Audio Test Program\n";
+  cout << "Audio Test Program\n\n";
 
   PSoundChannel::Directions dir;
   PStringArray namesPlay, namesRecord;
+  namesPlay = PSoundChannel::GetDeviceNames(PSoundChannel::Player);
+  namesRecord = PSoundChannel::GetDeviceNames(PSoundChannel::Recorder);
 
-  cout << "\n";
-  cout << "List of play devices\n";
-
-  dir = PSoundChannel::Player;
-  namesPlay = PSoundChannel::GetDeviceNames(dir);
-  for (PINDEX i = 0; i < namesPlay.GetSize(); i++)
-    cout << "  \"" << namesPlay[i] << "\"\n";
-
-  cout << "The default play device is \"" << PSoundChannel::GetDefaultDevice(dir) << "\"\n";
-
-
-  cout << "\n";
-  cout << "List of Record devices\n";
-
-  dir = PSoundChannel::Recorder;
-  namesRecord = PSoundChannel::GetDeviceNames(dir);
-  for (PINDEX i = 0; i < namesRecord.GetSize(); i++)
-    cout << "  \"" << namesRecord[i] << "\"\n";
-
-  cout << "The default record device is \"" << PSoundChannel::GetDefaultDevice(dir) << "\"\n";
-
-  cout << "\n";
-
+  if (args.HasOption('r')) {
+      cout << "List of play devices\n";
+      for (PINDEX i = 0; i < namesPlay.GetSize(); i++)
+	  cout << "  \"" << namesPlay[i] << "\"\n";      
+      cout << "The default play device is \"" 
+	   << PSoundChannel::GetDefaultDevice(PSoundChannel::Player) << "\"\n";
+      
+      cout << "List of Record devices\n";
+      for (PINDEX i = 0; i < namesRecord.GetSize(); i++)
+	  cout << "  \"" << namesRecord[i] << "\"\n";      
+      cout << "The default record device is \"" 
+	   << PSoundChannel::GetDefaultDevice(PSoundChannel::Recorder) << "\"\n\n";
+  }
 
   // Display the mixer settings for the default devices (or device if specified)
   {
@@ -121,28 +115,34 @@ void Audio::Main()
       devName = args.GetOptionString('s');
       if (devName.IsEmpty())
 	  devName = PSoundChannel::GetDefaultDevice(dir);
-      sound.Open(devName, dir);
-      
-      unsigned int vol;
-      if (sound.GetVolume(vol))
-	  cout << "Play volume is (for " << devName << ")" << vol << endl;
-      else
-	  cout << "Play volume cannot be obtained (for " << devName << ")" << endl;
-      
-      sound.Close();
       
+      if (sound.Open(devName, dir)) {     
+	  unsigned int vol;
+	  if (sound.GetVolume(vol))
+	      cout << "Play volume is (for " << devName << ")" << vol << endl;
+	  else
+	      cout << "Play volume cannot be obtained (for " << devName << ")" << endl;      
+	  sound.Close();
+      } else
+	  cerr << "Failed to open play device (" 
+	       << devName << ") to get volume settings" << endl;
+  }
+  {
+      PSoundChannel sound;
       dir = PSoundChannel::Recorder;
       devName = args.GetOptionString('s');
       if (devName.IsEmpty())
 	  devName = PSoundChannel::GetDefaultDevice(dir);
-      sound.Open(devName, dir);
-      
-      if (sound.GetVolume(vol))
-	  cout << "Record volume is (for " << devName << ")" << vol << endl;
-      else
-	  cout << "Record volume cannot be obtained (for " << devName << ")" << endl;
-      
-      sound.Close();
+      if (sound.Open(devName, dir)) {     
+	  unsigned int vol;
+	  if (sound.GetVolume(vol))
+	      cout << "Record volume is (for " << devName << ")" << vol << endl;
+	  else
+	      cout << "Record volume cannot be obtained (for " << devName << ")" << endl;
+	  sound.Close();
+      } else
+	  cerr << "Failed to open record device (" 
+	       << devName << ") to get volume settings" << endl;
   }
 
   if (args.HasOption('f')) {
@@ -169,6 +169,48 @@ void Audio::Main()
     return;
   }
 
+  if (args.HasOption('p')) {
+      PString playFileName = args.GetOptionString('p');
+      if (playFileName.IsEmpty()) {
+	  cerr << "The p option requires an arguement - the name of the file to play" << endl;
+	  cerr << "Terminating" << endl;
+	  return;
+      }
+
+      PWAVFile audioFile;
+      BYTE buffer[500];
+      if (!audioFile.Open(playFileName)) {
+	  cerr << "Failed to open the file \"" << playFileName << "\" to read audio from." << endl;
+	  return;
+      }
+
+      PSoundChannel sound;
+      dir = PSoundChannel::Player;
+      devName = args.GetOptionString('s');
+      if (devName.IsEmpty())
+	  devName = PSoundChannel::GetDefaultDevice(dir);
+      
+      if (sound.Open(devName, PSoundChannel::Player, 1, 8000, 16)) {
+	  PTRACE(3, "Open sound device " << devName << " to put audio to");
+	  cerr << devName << " opened fine for playing to" << endl;
+      } else {
+	  cerr << "Failed to open play device (" 
+	       << devName << ") for putting audio to speaker" << endl;
+	  return;
+      }
+      sound.SetBuffers(480, 3);
+      
+      PINDEX readCounter = 0;
+      while (audioFile.Read(buffer, 480)) {
+	  PTRACE(3, "Read buffer " << readCounter << " from audio file " << playFileName);
+	  readCounter++;
+	  if (!sound.Write(buffer, 480)) {
+	      cerr << " failed to write a buffer to sound device. Ending" << endl;
+	      return;
+	  }
+      }
+	  
+  }
 #if PTRACING
   if (args.GetOptionCount('t') > 0) {
     PTrace::ClearOptions(0);
@@ -319,6 +361,7 @@ TestAudioRead::TestAudioRead(TestAudioDevice &master, const PString & _captureFi
      captureFileName(_captureFileName)
 {    
   PTRACE(3, "Reader\tInitiate thread for reading " );
+  Resume();
 }
 
 void TestAudioRead::ReportIterations()
@@ -335,8 +378,10 @@ void TestAudioRead::Main()
     return;
   }
   PWAVFile audioFile;
-  if (!audioFile.Open(captureFileName, PFile::WriteOnly, PFile::Create | PFile::Truncate))
-      cerr << "Cannot create the file " << captureFileName << " to write audio to" << endl;
+  if (!captureFileName.IsEmpty()) {
+      if (!audioFile.Open(captureFileName, PFile::WriteOnly, PFile::Create | PFile::Truncate))
+	  cerr << "Cannot create the file " << captureFileName << " to write audio to" << endl;
+  }
 
   PTRACE(3, "TestAduioRead\tSound device is now open, start running");
 
@@ -363,6 +408,7 @@ TestAudioWrite::TestAudioWrite(TestAudioDevice &master)
    : TestAudio(master)
 {
   PTRACE(3, "Reader\tInitiate thread for writing " );
+  Resume();
 }
 
 void TestAudioWrite::ReportIterations()
@@ -401,7 +447,6 @@ TestAudio::TestAudio(TestAudioDevice &master)
 {
     iterations = 0;
     keepGoing = PTrue;
-    Resume();
 }
 
 TestAudio::~TestAudio()
@@ -459,5 +504,14 @@ void TestAudio::LowerVolume()
 }
 ////////////////////////////////////////////////////////////////////////////////
 
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 4 spaces.     
+ *
+ *
+ * Local Variables:
+ * mode:c
+ * c-basic-offset:4
+ * End:
+ */
 
 // End of hello.cxx
