@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24540 $
+ * $Revision: 25012 $
  * $Author: rjongbloed $
- * $Date: 2010-06-28 20:44:22 -0500 (Mon, 28 Jun 2010) $
+ * $Date: 2011-01-06 01:01:23 -0600 (Thu, 06 Jan 2011) $
  */
 
 #include <ptlib.h>
@@ -572,7 +572,7 @@ PString PHTTPClientAuthentication::GetAuthParam(const PString & auth, const char
 
   PINDEX pos = auth.Find(name);
   if (pos != P_MAX_INDEX)  {
-    pos += strlen(name);
+    pos += (int)strlen(name);
     while (isspace(auth[pos]) || (auth[pos] == ','))
       pos++;
     if (auth[pos] == '=') {
@@ -836,7 +836,7 @@ PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) con
          << "response=\"" << AsHex(response) << "\", "
          << "cnonce=\"" << cnonce << "\", "
          << "nc=" << nc << ", "
-         << "qop=" << qop;
+         << "qop=\"" << qop << '"';
   }
   else {
     digestor.Process(AsHex(a2));
@@ -909,6 +909,41 @@ PString PHTTPClientAuthenticator::GetMethod()
 
 ////////////////////////////////////////////////////////////////////////////////////
 
+#undef new
+
+class PURL_HttpLoader : public PURLLoader
+{
+    PCLASSINFO(PURL_HttpLoader, PURLLoader);
+  public:
+    virtual bool Load(const PURL & url, PString & str, const PString & requiredContentType)
+    {
+      PHTTPClient http;
+      return http.GetTextDocument(url, str, requiredContentType);
+    }
+
+    virtual bool Load(const PURL & url, PBYTEArray & data, const PString & requiredContentType)
+    {
+      PHTTPClient http;
+      PMIMEInfo outMIME, replyMIME;
+      if (!http.GetDocument(url, outMIME, replyMIME))
+        return false;
+
+      PCaselessString actualContentType = replyMIME(PHTTP::ContentTypeTag());
+      if (!requiredContentType.IsEmpty() && !actualContentType.IsEmpty() &&
+            actualContentType.NumCompare(requiredContentType, requiredContentType.Find(';')) != EqualTo) {
+        PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting " << requiredContentType << ", got " << actualContentType);
+        return false;
+      }
+
+      return http.ReadContentBody(replyMIME, data);
+    }
+};
+
+PFACTORY_CREATE(PURLLoaderFactory, PURL_HttpLoader, "http", true);
+#if P_SSL
+static PURLLoaderFactory::Worker<PURL_HttpLoader> httpsLoader("https", true);
+#endif
+
 #endif // P_HTTP
 
 
