@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20503 $
- * $Author: shorne $
- * $Date: 2008-06-26 12:52:56 +0000 (Thu, 26 Jun 2008) $
+ * $Revision: 25075 $
+ * $Author: willamowius $
+ * $Date: 2011-01-19 12:22:45 -0600 (Wed, 19 Jan 2011) $
  */
 
 #ifdef __GNUC__
@@ -47,7 +47,8 @@
 
 
 #if defined(_MSC_VER)
-#pragma comment(lib, P_LDAP_LIBRARY)
+  #pragma comment(lib, P_LDAP_LIBRARY)
+  #pragma message("LDAP support enabled")
 #endif
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -740,8 +741,13 @@ PBoolean PLDAPSession::GetNextSearchResult(SearchContext & context)
             errorNumber = LDAP_NO_RESULTS_RETURNED;
           context.completed = PTrue;
           return PFalse;
+        case LDAP_RES_SEARCH_REFERENCE :
+          errorNumber = LDAP_SUCCESS;
+          return PTrue;
+        // Ignore other result message types for now ...
+        default:
+          PTRACE(3, "Unhandled LDAP message type " << ldap_msgtype(context.message));
       }
-      // Ignore other result message types for now ...
     }
 
     ldap_msgfree(context.result);
@@ -1043,6 +1049,12 @@ void PLDAPSchema::OnSendSchema(PArray<PLDAPSession::ModAttrib> & attrib, PLDAPSe
 }
 
 
+#else
+
+  #if defined(_MSC_VER)
+    #pragma message("LDAP support DISABLED")
+  #endif
+
 #endif // P_LDAP
 
 
