@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23120 $
+ * $Revision: 25963 $
  * $Author: rjongbloed $
- * $Date: 2009-07-21 23:31:08 +0000 (Tue, 21 Jul 2009) $
+ * $Date: 2011-06-02 20:56:12 -0500 (Thu, 02 Jun 2011) $
  */
 
 #ifdef __GNUC__
@@ -39,8 +39,6 @@
 #define new PNEW
 
 
-PCREATE_NAT_PLUGIN(STUN);
-
 // Sample server is at larry.gloo.net
 
 #define DEFAULT_REPLY_TIMEOUT 800
@@ -48,6 +46,9 @@ PCREATE_NAT_PLUGIN(STUN);
 #define DEFAULT_NUM_SOCKETS_FOR_PAIRING 4
 
 
+typedef PSTUNClient PNatMethod_STUN;
+PCREATE_NAT_PLUGIN(STUN);
+
 ///////////////////////////////////////////////////////////////////////
 
 PSTUNClient::PSTUNClient()
@@ -477,9 +478,9 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(PBoolean force)
 
   PIPSocket::InterfaceTable interfaces;
   if (PIPSocket::GetInterfaceTable(interfaces)) {
-    for (PINDEX i =0; i < interfaces.GetSize(); i++) {
+    for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
       PIPSocket::Address binding = interfaces[i].GetAddress();
-      if (!binding.IsLoopback()) {
+      if (!binding.IsLoopback() && binding.GetVersion() == 4) {
         PUDPSocket * socket = new PUDPSocket;
         if (OpenSocket(*socket, singlePortInfo, binding))
           sockets.Append(socket);
