@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23915 $
+ * $Revision: 25925 $
  * $Author: rjongbloed $
- * $Date: 2010-01-10 21:12:43 -0600 (Sun, 10 Jan 2010) $
+ * $Date: 2011-05-30 20:40:07 -0500 (Mon, 30 May 2011) $
  */
 
 #ifdef __GNUC__
@@ -36,12 +36,10 @@
 
 #if P_URL
 
-#define P_DISABLE_FACTORY_INSTANCES
-#include <ptlib/pfactory.h>
-
 #include <ptclib/url.h>
 
 #include <ptlib/sockets.h>
+#include <ptclib/cypher.h>
 #include <ctype.h>
 
 #if defined(_WIN32) && !defined(_WIN32_WCE)
@@ -82,39 +80,29 @@
 #define DEFAULT_SIP_PORT      5060
 #define DEFAULT_SIPS_PORT     5061
 
-#define DEFINE_LEGACY_URL_SCHEME(schemeName, user, pass, host, def, defhost, query, params, frags, path, rel, port) \
-  class PURLLegacyScheme_##schemeName : public PURLLegacyScheme \
-  { \
-    public: \
-      PURLLegacyScheme_##schemeName() \
-        : PURLLegacyScheme(#schemeName, user, pass, host, def, defhost, query, params, frags, path, rel, port) \
-        { } \
-  }; \
-  static PFactory<PURLScheme>::Worker<PURLLegacyScheme_##schemeName> schemeName##Factory(#schemeName, true); \
 
-//                       schemeName,user,   passwd, host,   defUser,defhost, query,  params, frags,  path,   rel,    port
-DEFINE_LEGACY_URL_SCHEME(http,      PTrue,  PTrue,  PTrue,  PFalse, PTrue,   PTrue,  PTrue,  PTrue,  PTrue,  PTrue,  DEFAULT_HTTP_PORT )
-DEFINE_LEGACY_URL_SCHEME(file,      PFalse, PFalse, PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(https,     PFalse, PFalse, PTrue,  PFalse, PTrue,   PTrue,  PTrue,  PTrue,  PTrue,  PTrue,  DEFAULT_HTTPS_PORT)
-DEFINE_LEGACY_URL_SCHEME(gopher,    PFalse, PFalse, PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_GOPHER_PORT)
-DEFINE_LEGACY_URL_SCHEME(wais,      PFalse, PFalse, PTrue,  PFalse, PFalse,  PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_WAIS_PORT)
-DEFINE_LEGACY_URL_SCHEME(nntp,      PFalse, PFalse, PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_NNTP_PORT)
-DEFINE_LEGACY_URL_SCHEME(prospero,  PFalse, PFalse, PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_PROSPERO_PORT)
-DEFINE_LEGACY_URL_SCHEME(rtsp,      PFalse, PFalse, PTrue,  PFalse, PTrue,   PTrue,  PFalse, PFalse, PTrue,  PFalse, DEFAULT_RTSP_PORT)
-DEFINE_LEGACY_URL_SCHEME(rtspu,     PFalse, PFalse, PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_RTSPU_PORT)
-DEFINE_LEGACY_URL_SCHEME(ftp,       PTrue,  PTrue,  PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_FTP_PORT)
-DEFINE_LEGACY_URL_SCHEME(telnet,    PTrue,  PTrue,  PTrue,  PFalse, PTrue,   PFalse, PFalse, PFalse, PFalse, PFalse, DEFAULT_TELNET_PORT)
-DEFINE_LEGACY_URL_SCHEME(mailto,    PFalse, PFalse, PFalse, PFalse, PTrue,   PTrue,  PFalse, PFalse, PFalse, PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(news,      PFalse, PFalse, PFalse, PFalse, PTrue,   PFalse, PFalse, PFalse, PFalse, PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(h323,      PTrue,  PFalse, PTrue,  PTrue,  PFalse,  PFalse, PTrue,  PFalse, PFalse, PFalse, DEFAULT_H323_PORT)
-DEFINE_LEGACY_URL_SCHEME(h323s,     PTrue,  PFalse, PTrue,  PTrue,  PFalse,  PFalse, PTrue,  PFalse, PFalse, PFalse, DEFAULT_H323S_PORT)
-DEFINE_LEGACY_URL_SCHEME(rtmp,      PFalse, PFalse, PTrue,  PFalse, PFalse,  PFalse, PFalse, PFalse, PTrue,  PFalse, DEFAULT_RTMP_PORT)
-DEFINE_LEGACY_URL_SCHEME(sip,       PTrue,  PTrue,  PTrue,  PFalse, PFalse,  PTrue,  PTrue,  PFalse, PFalse, PFalse, DEFAULT_SIP_PORT)
-DEFINE_LEGACY_URL_SCHEME(sips,      PTrue,  PTrue,  PTrue,  PFalse, PFalse,  PTrue,  PTrue,  PFalse, PFalse, PFalse, DEFAULT_SIPS_PORT)
-DEFINE_LEGACY_URL_SCHEME(tel,       PFalse, PFalse, PFalse, PTrue,  PFalse,  PFalse, PTrue,  PFalse, PFalse, PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(fax,       PFalse, PFalse, PFalse, PTrue,  PFalse,  PFalse, PTrue,  PFalse, PFalse, PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(callto,    PFalse, PFalse, PFalse, PTrue,  PFalse,  PFalse, PTrue,  PFalse, PFalse, PFalse, 0)
-DEFINE_LEGACY_URL_SCHEME(msrp,      false,  false,  true,   false,  false,   true,   true,   false,  true,   false,  DEFAULT_MSRP_PORT)
+//                 schemeName,user,  passwd,host,  defUser,defhost,query, params,frags, path,  rel,   port
+PURL_LEGACY_SCHEME(http,      true,  true,  true,  false,  true,   true,  true,  true,  true,  true,  DEFAULT_HTTP_PORT )
+PURL_LEGACY_SCHEME(file,      false, false, true,  false,  true,   false, false, false, true,  false, 0)
+PURL_LEGACY_SCHEME(https,     false, false, true,  false,  true,   true,  true,  true,  true,  true,  DEFAULT_HTTPS_PORT)
+PURL_LEGACY_SCHEME(gopher,    false, false, true,  false,  true,   false, false, false, true,  false, DEFAULT_GOPHER_PORT)
+PURL_LEGACY_SCHEME(wais,      false, false, true,  false,  false,  false, false, false, true,  false, DEFAULT_WAIS_PORT)
+PURL_LEGACY_SCHEME(nntp,      false, false, true,  false,  true,   false, false, false, true,  false, DEFAULT_NNTP_PORT)
+PURL_LEGACY_SCHEME(prospero,  false, false, true,  false,  true,   false, false, false, true,  false, DEFAULT_PROSPERO_PORT)
+PURL_LEGACY_SCHEME(rtsp,      false, false, true,  false,  true,   true,  false, false, true,  false, DEFAULT_RTSP_PORT)
+PURL_LEGACY_SCHEME(rtspu,     false, false, true,  false,  true,   false, false, false, true,  false, DEFAULT_RTSPU_PORT)
+PURL_LEGACY_SCHEME(ftp,       true,  true,  true,  false,  true,   false, false, false, true,  false, DEFAULT_FTP_PORT)
+PURL_LEGACY_SCHEME(telnet,    true,  true,  true,  false,  true,   false, false, false, false, false, DEFAULT_TELNET_PORT)
+PURL_LEGACY_SCHEME(mailto,    false, false, false, true,   false,  true,  false, false, false, false, 0)
+PURL_LEGACY_SCHEME(news,      false, false, false, false,  true,   false, false, false, false, false, 0)
+PURL_LEGACY_SCHEME(h323,      true,  false, true,  true,   false,  false, true,  false, false, false, DEFAULT_H323_PORT)
+PURL_LEGACY_SCHEME(h323s,     true,  false, true,  true,   false,  false, true,  false, false, false, DEFAULT_H323S_PORT)
+PURL_LEGACY_SCHEME(rtmp,      false, false, true,  false,  false,  false, false, false, true,  false, DEFAULT_RTMP_PORT)
+PURL_LEGACY_SCHEME(sip,       true,  true,  true,  false,  false,  true,  true,  false, false, false, DEFAULT_SIP_PORT)
+PURL_LEGACY_SCHEME(sips,      true,  true,  true,  false,  false,  true,  true,  false, false, false, DEFAULT_SIPS_PORT)
+PURL_LEGACY_SCHEME(fax,       false, false, false, true,   false,  false, true,  false, false, false, 0)
+PURL_LEGACY_SCHEME(callto,    false, false, false, true,   false,  false, true,  false, false, false, 0)
+PURL_LEGACY_SCHEME(msrp,      false, false, true,  false,  false,  true,  true,  false, true,  false, DEFAULT_MSRP_PORT)
 
 #define DEFAULT_SCHEME "http"
 #define FILE_SCHEME    "file"
@@ -123,8 +111,7 @@ DEFINE_LEGACY_URL_SCHEME(msrp,      false,  false,  true,   false,  false,   tru
 // PURL
 
 PURL::PURL()
-  : //scheme(SchemeTable[DEFAULT_SCHEME].name),
-    scheme(DEFAULT_SCHEME),
+  : scheme(DEFAULT_SCHEME),
     port(0),
     portSupplied (PFalse),
     relativePath(PFalse)
@@ -244,7 +231,7 @@ PString PURL::TranslateString(const PString & str, TranslationType type)
       safeChars += ":@&=+$,|";   // Section 3.3
       break;
 
-    case QueryTranslation :
+    default :
       break;    // Section 3.4, no reserved characters may be used
   }
   PINDEX pos = (PINDEX)-1;
@@ -262,8 +249,7 @@ PString PURL::UntranslateString(const PString & str, TranslationType type)
 
   PINDEX pos;
   if (type == PURL::QueryTranslation) {
-    /* Even though RFC2396 never mentions this, and RFC1630 is quite vague
-       about it, a lot of things do it so we have to do it too */
+    /* Even though RFC2396 never mentions this, RFC1630 does. */
     pos = (PINDEX)-1;
     while ((pos = xlat.Find('+', pos+1)) != P_MAX_INDEX)
       xlat[pos] = ' ';
@@ -285,7 +271,7 @@ PString PURL::UntranslateString(const PString & str, TranslationType type)
 }
 
 
-void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, char sep2)
+void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, char sep2, TranslationType type)
 {
   vars.RemoveAll();
 
@@ -299,12 +285,12 @@ void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, cha
     if (sep2pos > sep1next)
       sep2pos = sep1next;
 
-    PCaselessString key = PURL::UntranslateString(str(sep1prev, sep2pos-1), PURL::QueryTranslation);
+    PCaselessString key = PURL::UntranslateString(str(sep1prev, sep2pos-1), type);
     if (!key) {
-      PString data = PURL::UntranslateString(str(sep2pos+1, sep1next-1), PURL::QueryTranslation);
+      PString data = PURL::UntranslateString(str(sep2pos+1, sep1next-1), type);
 
       if (vars.Contains(key))
-        vars.SetAt(key, vars[key] + ',' + data);
+        vars.SetAt(key, vars[key] + '\n' + data);
       else
         vars.SetAt(key, data);
     }
@@ -314,6 +300,32 @@ void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, cha
 }
 
 
+void PURL::OutputVars(ostream & strm,
+                      const PStringToString & vars,
+                      char sep0,
+                      char sep1,
+                      char sep2,
+                      TranslationType type)
+{
+  for (PINDEX i = 0; i < vars.GetSize(); i++) {
+    if (i > 0)
+      strm << sep1;
+    else if (sep0 != '\0')
+      strm << sep0;
+
+    PString key  = TranslateString(vars.GetKeyAt (i), type);
+    PString data = TranslateString(vars.GetDataAt(i), type);
+
+    if (key.IsEmpty())
+      strm << data;
+    else if (data.IsEmpty())
+      strm << key;
+    else
+      strm << key << sep2 << data;
+  }
+}
+
+
 PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
 {
   scheme.MakeEmpty();
@@ -329,33 +341,39 @@ PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
   queryVars.RemoveAll();
   m_contents.MakeEmpty();
 
+  if (cstr == NULL)
+    return false;
+
   // copy the string so we can take bits off it
   while (((*cstr & 0x80) == 0x00) && isspace(*cstr))
     cstr++;
   PString url = cstr;
-
-  // Character set as per RFC2396
-  PINDEX pos = 0;
-  while ( ((*cstr & 0x80) != 0x00) || isalnum(url[pos]) || url[pos] == '+' || url[pos] == '-' || url[pos] == '.')
-    pos++;
-
-  PString schemeName;
+  if (url.IsEmpty())
+    return false;
 
   // get information which tells us how to parse URL for this
   // particular scheme
   PURLScheme * schemeInfo = NULL;
 
-  // Determine if the URL has an explicit scheme
-  if (url[pos] == ':') {
-    // get the scheme information
-    schemeInfo = PFactory<PURLScheme>::CreateInstance(url.Left(pos));
-    if (schemeInfo != NULL)
-      url.Delete(0, pos+1);
+  // Character set as per RFC2396
+  //    scheme        = alpha *( alpha | digit | "+" | "-" | "." )
+  if (isalpha(url[0])) {
+    PINDEX pos = 1;
+    while (isalnum(url[pos]) || url[pos] == '+' || url[pos] == '-' || url[pos] == '.')
+      ++pos;
+
+    // Determine if the URL has an explicit scheme
+    if (url[pos] == ':') {
+      // get the scheme information
+      schemeInfo = PURLSchemeFactory::CreateInstance(url.Left(pos));
+      if (schemeInfo != NULL)
+        url.Delete(0, pos+1);
+    }
   }
 
   // if we could not match a scheme, then use the specified default scheme
   if (schemeInfo == NULL && defaultScheme != NULL) {
-    schemeInfo = PFactory<PURLScheme>::CreateInstance(defaultScheme);
+    schemeInfo = PURLSchemeFactory::CreateInstance(defaultScheme);
     PAssert(schemeInfo != NULL, "Default scheme " + PString(defaultScheme) + " not available");
   }
 
@@ -546,7 +564,7 @@ PBoolean PURL::LegacyParse(const PString & _url, const PURLLegacyScheme * scheme
     // chop off any trailing parameters
     pos = url.Find(';');
     if (pos != P_MAX_INDEX) {
-      SplitVars(url(pos+1, P_MAX_INDEX), paramVars, ';', '=');
+      SplitVars(url(pos+1, P_MAX_INDEX), paramVars);
       url.Delete(pos, P_MAX_INDEX);
     }
   }
@@ -615,9 +633,9 @@ PString PURL::AsString(UrlFormat fmt) const
   if (scheme.IsEmpty())
     return PString::Empty();
 
-  const PURLScheme * schemeInfo = PFactory<PURLScheme>::CreateInstance(scheme);
+  const PURLScheme * schemeInfo = PURLSchemeFactory::CreateInstance(scheme);
   if (schemeInfo == NULL)
-    schemeInfo = PFactory<PURLScheme>::CreateInstance(DEFAULT_SCHEME);
+    schemeInfo = PURLSchemeFactory::CreateInstance(DEFAULT_SCHEME);
 
   return schemeInfo->AsString(fmt, *this);
 }
@@ -625,7 +643,6 @@ PString PURL::AsString(UrlFormat fmt) const
 PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schemeInfo) const
 {
   PStringStream str;
-  PINDEX i;
 
   if (fmt == HostPortOnly) {
     str << scheme << ':';
@@ -684,15 +701,8 @@ PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schem
     if (!fragment)
       str << "#" << TranslateString(fragment, PathTranslation);
 
-    for (i = 0; i < paramVars.GetSize(); i++) {
-      str << ';' << TranslateString(paramVars.GetKeyAt(i), QueryTranslation);
-      PString data = paramVars.GetDataAt(i);
-      if (!data)
-        str << '=' << TranslateString(data, QueryTranslation);
-    }
-
-    if (!queryVars.IsEmpty())
-      str << '?' << GetQuery();
+    OutputVars(str, paramVars, ';', ';', '=', ParameterTranslation);
+    OutputVars(str, queryVars, '?', '&', '=', QueryTranslation);
   }
 
   return str;
@@ -782,24 +792,15 @@ void PURL::AppendPath(const PString & segment)
 
 PString PURL::GetParameters() const
 {
-  PStringStream str;
-
-  for (PINDEX i = 0; i < paramVars.GetSize(); i++) {
-    if (i > 0)
-      str << ';';
-    str << TranslateString(paramVars.GetKeyAt(i), QueryTranslation);
-    PString data = paramVars.GetDataAt(i);
-    if (!data)
-      str << '=' << TranslateString(data, QueryTranslation);
-  }
-
-  return str;
+  PStringStream strm;
+  OutputVars(strm, paramVars, '\0', ';', '=', ParameterTranslation);
+  return strm;
 }
 
 
 void PURL::SetParameters(const PString & parameters)
 {
-  SplitVars(parameters, paramVars, ';', '=');
+  SplitVars(parameters, paramVars);
   Recalculate();
 }
 
@@ -823,22 +824,9 @@ void PURL::SetParamVar(const PString & key, const PString & data, bool emptyData
 
 PString PURL::GetQuery() const
 {
-  PStringStream str;
-
-  for (PINDEX i = 0; i < queryVars.GetSize(); i++) {
-    if (i > 0)
-      str << '&';
-    PString key  = TranslateString(queryVars.GetKeyAt (i), QueryTranslation);
-    PString data = TranslateString(queryVars.GetDataAt(i), QueryTranslation);
-    if (key.IsEmpty())
-      str << data;
-    else if (data.IsEmpty())
-      str << key;
-    else
-      str << key << '=' << data;
-  }
-
-  return str;
+  PStringStream strm;
+  OutputVars(strm, queryVars, '\0', '&', '=', QueryTranslation);
+  return strm;
 }
 
 
@@ -873,7 +861,21 @@ void PURL::SetContents(const PString & str)
 }
 
 
-PBoolean PURL::OpenBrowser(const PString & url)
+bool PURL::LoadResource(PString & str, const PString & requiredContentType)
+{
+  PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
+  return loader != NULL && loader->Load(*this, str, requiredContentType);
+}
+
+
+bool PURL::LoadResource(PBYTEArray & data, const PString & requiredContentType)
+{
+  PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
+  return loader != NULL && loader->Load(*this, data, requiredContentType);
+}
+
+
+bool PURL::OpenBrowser(const PString & url)
 {
 #ifdef _WIN32
   SHELLEXECUTEINFO sei;
@@ -884,14 +886,14 @@ PBoolean PURL::OpenBrowser(const PString & url)
   sei.lpFile = file;
 
   if (ShellExecuteEx(&sei) != 0)
-    return PTrue;
+    return true;
 
   PVarString msg = "Unable to open page" & url;
   PVarString name = PProcess::Current().GetName();
   MessageBox(NULL, msg, name, MB_TASKMODAL);
 
 #endif // WIN32
-  return PFalse;
+  return false;
 }
 
 
@@ -903,6 +905,209 @@ void PURL::Recalculate()
   urlString = AsString(HostPortOnly) + AsString(URIOnly);
 }
 
+
+///////////////////////////////////////////////////////////////////////////////
+
+// RFC3966 tel URI
+
+class PURL_TelScheme : public PURLScheme
+{
+    PCLASSINFO(PURL_TelScheme, PURLScheme);
+  public:
+    virtual PString GetName() const
+    {
+      return "tel";
+    }
+
+    virtual PBoolean Parse(const PString & str, PURL & url) const
+    {
+      PINDEX pos = str.FindSpan("0123456789*#", str[0] != '+' ? 0 : 1);
+      if (pos == P_MAX_INDEX)
+        url.SetUserName(str);
+      else {
+        if (str[pos] != ';')
+          return false;
+
+        url.SetUserName(str.Left(pos));
+
+        PStringToString paramVars;
+        PURL::SplitVars(str(pos+1, P_MAX_INDEX), paramVars);
+        url.SetParamVars(paramVars);
+
+        PString phoneContext = paramVars("phone-context");
+        if (phoneContext.IsEmpty()) {
+          if (str[0] != '+')
+            return false;
+        }
+        else if (phoneContext[0] != '+')
+          url.SetHostName(phoneContext);
+        else if (str[0] != '+')
+          url.SetUserName(phoneContext+url.GetUserName());
+        else
+          return false;
+      }
+
+      return url.GetUserName() != "+";
+    }
+
+    virtual PString AsString(PURL::UrlFormat fmt, const PURL & url) const
+    {
+      if (fmt == PURL::HostPortOnly)
+        return PString::Empty();
+
+      PStringStream strm;
+      strm << "tel:" + url.GetUserName();
+      PURL::OutputVars(strm, url.GetParamVars(), ';', ';', '=', PURL::ParameterTranslation);
+      return strm;
+    }
+};
+
+static PURLSchemeFactory::Worker<PURL_TelScheme> telScheme("tel", true);
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+// RFC2397 data URI
+
+class PURL_DataScheme : public PURLScheme
+{
+    PCLASSINFO(PURL_DataScheme, PURLScheme);
+  public:
+    virtual PString GetName() const
+    {
+      return "data";
+    }
+
+    virtual PBoolean Parse(const PString & url, PURL & purl) const
+    {
+      PINDEX comma = url.Find(',');
+      if (comma == P_MAX_INDEX)
+        return false;
+
+      PINDEX semi = url.Find(';');
+      if (semi > comma)
+        purl.SetParamVar("type", url.Left(comma));
+      else {
+        purl.SetParameters(url(semi, comma-1));
+        purl.SetParamVar("type", url.Left(semi));
+      }
+
+      purl.SetContents(url.Mid(comma+1));
+
+      return true;
+    }
+
+    virtual PString AsString(PURL::UrlFormat fmt, const PURL & purl) const
+    {
+      if (fmt == PURL::HostPortOnly)
+        return PString::Empty();
+
+      const PStringToString & params = purl.GetParamVars();
+      PStringStream strm;
+
+      strm << "data:" + params("type", "text/plain");
+
+      bool base64 = false;
+      for (PINDEX i = 0; i < params.GetSize(); i++) {
+        PCaselessString key = params.GetKeyAt(i);
+        if (key == "type")
+          continue;
+        if (key == "base64") {
+          base64 = true;
+          continue;
+        }
+
+        strm << ';' << PURL::TranslateString(key, PURL::ParameterTranslation);
+
+        PString data = params.GetDataAt(i);
+        if (!data)
+          strm << '=' << PURL::TranslateString(data, PURL::ParameterTranslation);
+      }
+
+      // This must always be last according to EBNF
+      if (base64)
+        strm << ";base64";
+
+      strm << ',' << PURL::TranslateString(purl.GetContents(), PURL::ParameterTranslation);
+
+      return strm;
+    }
+};
+
+static PURLSchemeFactory::Worker<PURL_DataScheme> dataScheme("data", true);
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+class PURL_FileLoader : public PURLLoader
+{
+    PCLASSINFO(PURL_FileLoader, PURLLoader);
+  public:
+    virtual bool Load(const PURL & url, PString & str, const PString &)
+    {
+      PTextFile file;
+      if (!file.Open(url.AsFilePath()))
+        return false;
+      if (!str.SetSize(file.GetLength()+1))
+        return false;
+      return file.Read(str.GetPointer(), str.GetSize()-1);
+    }
+
+    virtual bool Load(const PURL & url, PBYTEArray & data, const PString &)
+    {
+      PFile file;
+      if (!file.Open(url.AsFilePath()))
+        return false;
+      if (!data.SetSize(file.GetLength()))
+        return false;
+      return file.Read(data.GetPointer(), data.GetSize());
+    }
+};
+
+PFACTORY_CREATE(PURLLoaderFactory, PURL_FileLoader, "file", true);
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+class PURL_DataLoader : public PURLLoader
+{
+    PCLASSINFO(PURL_FileLoader, PURLLoader);
+  public:
+    virtual bool Load(const PURL & url, PString & str, const PString & requiredContentType)
+    {
+      if (!requiredContentType.IsEmpty()) {
+        PCaselessString actualContentType = url.GetParamVars()("type");
+        if (!actualContentType.IsEmpty() && requiredContentType != requiredContentType)
+          return false;
+      }
+
+      str = url.GetContents();
+      return true;
+    }
+
+    virtual bool Load(const PURL & url, PBYTEArray & data, const PString & requiredContentType)
+    {
+      if (!requiredContentType.IsEmpty()) {
+        PCaselessString actualContentType = url.GetParamVars()("type");
+        if (!actualContentType.IsEmpty() && requiredContentType != requiredContentType)
+          return false;
+      }
+
+      if (url.GetParamVars().Contains("base64"))
+        return PBase64::Decode(url.GetContents(), data);
+
+      PString str = url.GetContents();
+      PINDEX len = str.GetLength();
+      if (!data.SetSize(len))
+        return false;
+
+      memcpy(data.GetPointer(), (const char *)str, len);
+      return true;
+    }
+};
+
+PFACTORY_CREATE(PURLLoaderFactory, PURL_DataLoader, "data", true);
+
 #endif // P_URL
 
 
