@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24372 $
+ * $Revision: 25392 $
  * $Author: rjongbloed $
- * $Date: 2010-05-24 00:42:36 -0500 (Mon, 24 May 2010) $
+ * $Date: 2011-03-22 23:29:41 -0500 (Tue, 22 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -68,6 +68,7 @@ PContainer::PContainer(PINDEX initialSize)
   PAssert(reference != NULL, POutOfMemory);
 }
 
+
 PContainer::PContainer(int, const PContainer * cont)
 {
   if (cont == this)
@@ -80,6 +81,7 @@ PContainer::PContainer(int, const PContainer * cont)
   PAssert(reference != NULL, POutOfMemory);
 }
 
+
 PContainer::PContainer(const PContainer & cont)
 {
   if (&cont == this)
@@ -92,24 +94,29 @@ PContainer::PContainer(const PContainer & cont)
 }
 
 
+PContainer::PContainer(PContainerReference & ref)
+  : reference(&ref)
+{
+}
+
+
 void PContainer::AssignContents(const PContainer & cont)
 {
   if(cont.reference == NULL){
     PAssertAlways("container reference is null");
     return;
-  } else if(cont.GetClass() == NULL){
+  }
+  if(cont.GetClass() == NULL){
     PAssertAlways("container class is null");
     return;
   }
 
-  if (reference == cont.reference) {
+  if (reference == cont.reference)
     return;
-  }
 
   if (--reference->count == 0) {
     DestroyContents();
-    delete reference;
-    reference = NULL;
+    DestroyReference();
   }
 
   PAssert(++cont.reference->count > 1, "Assignment of container that was deleted");
@@ -120,19 +127,21 @@ void PContainer::AssignContents(const PContainer & cont)
 void PContainer::Destruct()
 {
   if (reference != NULL) {
-    if (--reference->count > 0) {
-      reference = NULL;
-    }
-    
-    else {
+    if (--reference->count <= 0) {
       DestroyContents();
-      delete reference;
-      reference = NULL;
+      DestroyReference();
     }
+    reference = NULL;
   }
 }
 
 
+void PContainer::DestroyReference()
+{
+  delete reference;
+}
+
+
 PBoolean PContainer::SetMinSize(PINDEX minSize)
 {
   PASSERTINDEX(minSize);
@@ -203,6 +212,15 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
 }
 
 
+PAbstractArray::PAbstractArray(PContainerReference & reference, PINDEX elementSizeInBytes)
+  : PContainer(reference)
+  , elementSize(elementSizeInBytes)
+  , theArray(NULL)
+  , allocatedDynamically(false)
+{
+}
+
+
 void PAbstractArray::DestroyContents()
 {
   if (theArray != NULL) {
@@ -218,6 +236,9 @@ void PAbstractArray::CopyContents(const PAbstractArray & array)
   elementSize = array.elementSize;
   theArray = array.theArray;
   allocatedDynamically = array.allocatedDynamically;
+
+  if (reference->constObject)
+    MakeUnique();
 }
 
 
@@ -325,6 +346,8 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
       if ((newArray = PAbstractArray_allocator.allocate(newsizebytes)) == NULL)
         return PFalse;
   
+      allocatedDynamically = true;
+
       if (theArray != NULL)
         memcpy(newArray, theArray, PMIN(oldsizebytes, newsizebytes));
     }
@@ -592,7 +615,7 @@ PBoolean PBitArray::Concatenate(const PBitArray & array)
 ///////////////////////////////////////////////////////////////////////////////
 
 PString::PString(const char * cstr)
-  : PCharArray(cstr != NULL ? strlen(cstr)+1 : 1)
+  : PCharArray(cstr != NULL ? (int)strlen(cstr)+1 : 1)
 {
   if (cstr != NULL)
     memcpy(theArray, cstr, GetSize());
@@ -2575,6 +2598,56 @@ char ** PStringToString::ToCharArray(bool withEqualSign, PCharArray * storage) c
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+
+PString PStringOptions::GetString(const PCaselessString & key, const char * dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? *str : PString(dflt);
+}
+
+
+bool PStringOptions::GetBoolean(const PCaselessString & key, bool dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  if (str == NULL)
+    return dflt;
+
+  if (str->IsEmpty() || str->AsUnsigned() != 0)
+    return true;
+
+  PCaselessString test(*str);
+  return test.NumCompare("t") == EqualTo || test.NumCompare("y") == EqualTo;
+}
+
+
+long PStringOptions::GetInteger(const PCaselessString & key, long dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? str->AsInteger() : dflt;
+}
+
+
+void PStringOptions::SetInteger(const PCaselessString & key, long value)
+{
+  PStringToString::SetAt(key, PString(PString::Signed, value));
+}
+
+
+double PStringOptions::GetReal(const PCaselessString & key, double dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? str->AsReal() : dflt;
+}
+
+
+void PStringOptions::SetReal(const PCaselessString & key, double value, int decimals)
+{
+  PStringToString::SetAt(key, PString(decimals < 0 ? PString::Exponent : PString::Decimal, value, decimals));
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PRegularExpression::PRegularExpression()
@@ -2629,6 +2702,12 @@ PRegularExpression::~PRegularExpression()
 }
 
 
+void PRegularExpression::PrintOn(ostream &strm) const
+{
+  strm << patternSaved;
+}
+
+
 PRegularExpression::ErrorCodes PRegularExpression::GetErrorCode() const
 {
   return lastError;
