@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24659 $
+ * $Revision: 25115 $
  * $Author: rjongbloed $
- * $Date: 2010-08-12 10:42:07 -0500 (Thu, 12 Aug 2010) $
+ * $Date: 2011-02-15 04:27:40 -0600 (Tue, 15 Feb 2011) $
  */
 
 #include <ptlib.h>
@@ -303,6 +303,11 @@ PObject::Comparison PTime::Compare(const PObject & obj) const
 }
 
 
+void PTime::SetTimestamp(time_t seconds, long usecs)
+{ 
+  microseconds = usecs%1000000;
+  theTime = seconds + usecs/1000000;
+}
 
 
 PString PTime::AsString(TimeFormat format, int zone) const
@@ -313,10 +318,14 @@ PString PTime::AsString(TimeFormat format, int zone) const
   switch (format) {
     case RFC1123 :
       return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
+    case RFC3339 :
+      return AsString("yyyy-MM-ddThh:mm:ssZZ", zone);
     case ShortISO8601 :
       return AsString("yyyyMMddThhmmssZ", zone);
     case LongISO8601 :
       return AsString("yyyy-MM-dd T hh:mm:ss Z", zone);
+    case EpochTime :
+      return psprintf("%u.%06lu", theTime, microseconds);
     default :
       break;
   }
@@ -528,14 +537,19 @@ PString PTime::AsString(const char * format, int zone) const
             str << 'Z';
           else
             str << "GMT";
+          while (toupper(*++format) == 'z')
+            ;
         }
         else {
+          while (toupper(*++format) == 'z')
+            repeatCount++;
           str << (zone < 0 ? '-' : '+');
           zone = PABS(zone);
-          str << setw(2) << (zone/60) << setw(2) << (zone%60);
+          str << setw(2) << (zone/60);
+          if (repeatCount > 0)
+            str << ':';
+          str << setw(2) << (zone%60);
         }
-        while (toupper(*++format) == 'z')
-          ;
         break;
 
       case 'u' :
