@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24142 $
+ * $Revision: 24979 $
  * $Author: rjongbloed $
- * $Date: 2010-03-22 03:01:34 -0500 (Mon, 22 Mar 2010) $
+ * $Date: 2010-12-22 05:34:45 -0600 (Wed, 22 Dec 2010) $
  */
 
 #ifdef __GNUC__
@@ -386,6 +386,30 @@ PINDEX PSafeCollection::GetSize() const
 }
 
 
+void PSafeCollection::CopySafeCollection(PCollection * other)
+{
+  DisallowDeleteObjects();
+
+  for (PINDEX i = 0; i < other->GetSize(); ++i) {
+    PSafeObject * obj = dynamic_cast<PSafeObject *>(other->GetAt(i));
+    if (obj != NULL && obj->SafeReference())
+      collection->Append(obj);
+  }
+}
+
+
+void PSafeCollection::CopySafeDictionary(PAbstractDictionary * other)
+{
+  DisallowDeleteObjects();
+
+  for (PINDEX i = 0; i < other->GetSize(); ++i) {
+    PSafeObject * obj = dynamic_cast<PSafeObject *>(&other->AbstractGetDataAt(i));
+    if (obj != NULL && obj->SafeReference())
+      collection->Insert(other->AbstractGetKeyAt(i), obj);
+  }
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 PSafePtrBase::PSafePtrBase(PSafeObject * obj, PSafetyMode mode)
