@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22284 $
+ * $Revision: 25667 $
  * $Author: rjongbloed $
- * $Date: 2009-03-25 09:44:02 +0000 (Wed, 25 Mar 2009) $
+ * $Date: 2011-05-04 00:32:36 -0500 (Wed, 04 May 2011) $
  */
 
 #include <ptlib.h>
-
+#include <ptbuildopts.h>
 #include <ptlib/sockets.h>
 
 #include <ctype.h>
@@ -67,12 +67,14 @@ void CALLBACK CompletionRoutine(DWORD dwError,
 #endif // P_QOS
 
 
-#if P_HAS_IPV6
-#define HAS_GETADDRINFO 1
-#endif
-
-#if (defined(_WIN32) && defined (_MSC_VER))
-#define HAS_GETADDRINFO 1
+#if !defined(P_MINGW) && !defined(P_CYGWIN)
+  #if P_HAS_IPV6 || defined(AI_NUMERICHOST)
+    #define HAS_GETADDRINFO 1
+  #endif
+#else
+  #if WINVER > 0x500
+    #define HAS_GETADDRINFO 1
+  #endif
 #endif
 
 
@@ -83,14 +85,18 @@ static int defaultIpAddressFamily = PF_INET;  // PF_UNSPEC;   // default to IPV4
 
 static PIPSocket::Address loopback4(127,0,0,1);
 static PIPSocket::Address broadcast4(INADDR_BROADCAST);
-static PIPSocket::Address any4(INADDR_ANY);
+static PIPSocket::Address any4(0,0,0,0);
 static in_addr inaddr_empty;
 #if P_HAS_IPV6
 static int defaultIPv6ScopeId = 0; 
 
 static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
 static PIPSocket::Address broadcast6(16,(const BYTE *)"\377\002\0\0\0\0\0\0\0\0\0\0\0\0\0\001"); // IPV6 multicast address
-static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"); 
+static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0");
+
+#define IPV6_PARAM(p) p
+#else
+#define IPV6_PARAM(p)
 #endif
 
 
@@ -162,32 +168,37 @@ PIPSocket::Address PIPSocket::GetDefaultIpAny()
 class Psockaddr
 {
   public:
-    Psockaddr() { memset(&storage, 0, sizeof(storage)); }
+    Psockaddr() : ptr(&storage) { memset(&storage, 0, sizeof(storage)); }
     Psockaddr(const PIPSocket::Address & ip, WORD port);
-    sockaddr* operator->() const { return (sockaddr *)&storage; }
-    operator sockaddr*()   const { return (sockaddr *)&storage; }
+    sockaddr* operator->() const { return addr; }
+    operator sockaddr*()   const { return addr; }
     socklen_t GetSize() const;
     PIPSocket::Address GetIP() const;
     WORD GetPort() const;
   private:
     sockaddr_storage storage;
+    union {
+      sockaddr_storage * ptr;
+      sockaddr         * addr;
+      sockaddr_in      * addr4;
+      sockaddr_in6     * addr6;
+    };
 };
 
 
 Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
+ : ptr(&storage) 
 {
   memset(&storage, 0, sizeof(storage));
 
   if (ip.GetVersion() == 6) {
-    sockaddr_in6 * addr6 = (sockaddr_in6 *)&storage;
     addr6->sin6_family = AF_INET6;
     addr6->sin6_addr = ip;
     addr6->sin6_port = htons(port);
     addr6->sin6_flowinfo = 0;
-	addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
+    addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
   }
   else {
-    sockaddr_in * addr4 = (sockaddr_in *)&storage;
     addr4->sin_family = AF_INET;
     addr4->sin_addr = ip;
     addr4->sin_port = htons(port);
@@ -197,7 +208,7 @@ Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
 
 socklen_t Psockaddr::GetSize() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
       return sizeof(sockaddr_in);
     case AF_INET6 :
@@ -212,11 +223,11 @@ socklen_t Psockaddr::GetSize() const
 
 PIPSocket::Address Psockaddr::GetIP() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
-      return ((sockaddr_in *)&storage)->sin_addr;
+      return addr4->sin_addr;
     case AF_INET6 :
-      return ((sockaddr_in6 *)&storage)->sin6_addr;
+      return addr6->sin6_addr;
     default :
       return 0;
   }
@@ -225,11 +236,11 @@ PIPSocket::Address Psockaddr::GetIP() const
 
 WORD Psockaddr::GetPort() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
-      return ntohs(((sockaddr_in *)&storage)->sin_port);
+      return ntohs(addr4->sin_port);
     case AF_INET6 :
-      return ntohs(((sockaddr_in6 *)&storage)->sin6_port);
+      return ntohs(addr6->sin6_port);
     default :
       return 0;
   }
@@ -283,7 +294,7 @@ class PHostByName : PHostByName_private
   friend void PIPSocket::ClearNameCache();
 };
 
-PMutex creationMutex;
+static PMutex creationMutex;
 static PHostByName & pHostByName()
 {
   PWaitAndSignal m(creationMutex);
@@ -534,6 +545,10 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     struct addrinfo *res = NULL;
     struct addrinfo hints = { AI_CANONNAME, defaultIpAddressFamily };
     localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
+    if (localErrNo != 0 && defaultIpAddressFamily == AF_INET6) {
+      hints.ai_family = AF_INET;
+      localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
+    }
     host = new PIPCacheData(localErrNo != NETDB_SUCCESS ? NULL : res, name);
     if (res != NULL)
       freeaddrinfo(res);
@@ -1503,34 +1518,30 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
   Psockaddr bind_sa(bindAddr, port); 
 #endif
 
-  if (!IsOpen()) {
-    // attempt to create a socket
 #if P_HAS_IPV6
-    if (!OpenSocket(bind_sa->sa_family))
-      return PFalse;
+  // Always close and re-open as the bindAddr address family might change.
+  Close();
+
+  // attempt to create a socket
+  if (!OpenSocket(bind_sa->sa_family))
+    return PFalse;
 #else
+  if (!IsOpen()) {
     if (!OpenSocket())
       return PFalse;
-#endif
   }
+#endif
   
 #ifndef P_BEOS
 #if P_HAS_IPV6
   if(bind_sa->sa_family != AF_INET6)
-  {
-	 // attempt to listen
-	 if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
-	   os_close();
-	   return PFalse;
-	 }
-  }
 #endif
-#else
-  // attempt to listen
-  int value = reuse == CanReuseAddress ? 1 : 0;
-  if (!SetOption(SO_REUSEADDR, &value, sizeof(int))) {
-    os_close();
-    return PFalse;
+  {
+   // attempt to listen
+   if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
+     os_close();
+     return PFalse;
+   }
   }
 #endif // BEOS
 
@@ -1589,7 +1600,7 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
     if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
       port = ntohs(sin.sin_port);
 
-	  if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
+    if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
         return true;
 
       struct ip_mreq  mreq;
@@ -1612,7 +1623,7 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 
 #if P_HAS_IPV6
 
-/// Check for v4 mapped i nv6 address ::ffff:a.b.c.d
+/// Check for v4 mapped in v6 address ::ffff:a.b.c.d
 PBoolean PIPSocket::Address::IsV4Mapped() const
 {
   if (version != 6)
@@ -1620,41 +1631,43 @@ PBoolean PIPSocket::Address::IsV4Mapped() const
   return IN6_IS_ADDR_V4MAPPED(&v.six) || IN6_IS_ADDR_V4COMPAT(&v.six);
 }
 
-
-const PIPSocket::Address & PIPSocket::Address::GetLoopback(int version)
+/// Check for link-local address fe80::/10
+PBoolean PIPSocket::Address::IsLinkLocal() const
 {
-  return version == 6 ? loopback6 : loopback4;
-}
-
-const PIPSocket::Address & PIPSocket::Address::GetAny(int version)
-{
-  return version == 6 ? any6 : any4;
-}
-
-const PIPSocket::Address PIPSocket::Address::GetBroadcast(int version)
-{
-  return version == 6 ? broadcast6 : broadcast4;
-}
-
-#else
-
-const PIPSocket::Address & PIPSocket::Address::GetLoopback(int)
-{
-  return loopback4;
-}
-
-const PIPSocket::Address & PIPSocket::Address::GetAny(int)
-{
-  return any4;
-}
-
-const PIPSocket::Address PIPSocket::Address::GetBroadcast(int)
-{
-  return broadcast4;
+  if (version != 6)
+    return PFalse;
+  return IN6_IS_ADDR_LINKLOCAL(&v.six);
 }
 
 #endif
 
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int IPV6_PARAM(version))
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return loopback6;
+#endif
+  return loopback4;
+}
+
+const PIPSocket::Address & PIPSocket::Address::GetAny(int IPV6_PARAM(version))
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return any6;
+#endif
+  return any4;
+}
+
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int IPV6_PARAM(version))
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return broadcast6;
+#endif
+  return broadcast4;
+}
+
 
 PBoolean PIPSocket::Address::IsAny() const
 {
@@ -1665,11 +1678,11 @@ PBoolean PIPSocket::Address::IsAny() const
 PIPSocket::Address::Address()
 {
 #if P_HAS_IPV6
-	if(defaultIpAddressFamily == AF_INET6)
-		*this = loopback6;
-	else
+  if(defaultIpAddressFamily == AF_INET6)
+    *this = loopback6;
+  else
 #endif
-		*this = loopback4;
+    *this = loopback4;
 }
 
 
@@ -1859,7 +1872,7 @@ PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 }
 
 
-PString PIPSocket::Address::AsString(bool bracketIPv6) const
+PString PIPSocket::Address::AsString(bool IPV6_PARAM(bracketIPv6)) const
 {
 #if defined(P_VXWORKS)
   char ipStorage[INET_ADDR_LEN];
@@ -1877,7 +1890,7 @@ PString PIPSocket::Address::AsString(bool bracketIPv6) const
     str.MakeMinimumSize();
     if (bracketIPv6)
       return '[' + str + ']';
-	return str;
+    return str;
   }
 #endif // P_HAS_IPV6
 # if defined(P_HAS_INET_NTOP)
@@ -2129,18 +2142,11 @@ PIPSocket::InterfaceEntry::InterfaceEntry()
 PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
                                           const Address & _addr,
                                           const Address & _mask,
-                                          const PString & _macAddr
-#if P_HAS_IPV6
-                                         ,const PString & _ip6Addr
-#endif
-                                         )
+                                          const PString & _macAddr)
   : name(_name.Trim()),
     ipAddr(_addr),
     netMask(_mask),
     macAddr(_macAddr)
-#if P_HAS_IPV6
-    , ip6Addr(_ip6Addr)
-#endif
 {
 }
 
@@ -2148,10 +2154,6 @@ PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
 void PIPSocket::InterfaceEntry::PrintOn(ostream & strm) const
 {
   strm << ipAddr;
-#if P_HAS_IPV6
-  if (!ip6Addr)
-    strm << " [" << ip6Addr << ']';
-#endif
   if (!macAddr)
     strm << " <" << macAddr << '>';
   if (!name)
@@ -2421,23 +2423,21 @@ PBoolean PIPDatagramSocket::ReadFrom(void * buf, PINDEX len,
 
   Psockaddr sa;
   PINDEX size = sa.GetSize();
-  if (os_recvfrom(buf, len, 0, sa, &size)) {
-    addr = sa.GetIP();
-    port = sa.GetPort();
-  }
+  bool ok = os_recvfrom(buf, len, 0, sa, &size);
+  addr = sa.GetIP();
+  port = sa.GetPort();
 
 #else
 
   sockaddr_in sockAddr;
   PINDEX addrLen = sizeof(sockAddr);
-  if (os_recvfrom(buf, len, 0, (struct sockaddr *)&sockAddr, &addrLen)) {
-    addr = sockAddr.sin_addr;
-    port = ntohs(sockAddr.sin_port);
-  }
+  bool ok = os_recvfrom(buf, len, 0, (struct sockaddr *)&sockAddr, &addrLen);
+  addr = sockAddr.sin_addr;
+  port = ntohs(sockAddr.sin_port);
 
 #endif
 
-  return lastReadCount > 0;
+  return ok;
 }
 
 
@@ -2604,11 +2604,8 @@ PQoS & PUDPSocket::GetQoSSpec()
 
 PBoolean PUDPSocket::ApplyQoS()
 {
-#ifdef _WIN32_WCE
-  return PFalse;   //QoS not supported
-#endif
-
   char DSCPval = 0;
+#ifndef _WIN32_WCE
   if (qosSpec.GetDSCP() < 0 ||
       qosSpec.GetDSCP() > 63) {
     if (qosSpec.GetServiceType() == SERVICETYPE_PNOTDEFINED)
@@ -2630,12 +2627,17 @@ PBoolean PUDPSocket::ApplyQoS()
   }
   else
     DSCPval = (char)qosSpec.GetDSCP();
+#else
+  DSCPval = 0x38;
+  disableGQoS = PFalse;
+#endif
 
 #ifdef _WIN32
 #if P_QOS
   if (disableGQoS)
     return PFalse;
 
+#ifndef _WIN32_WCE
   PBoolean usesetsockopt = PFalse;
 
   OSVERSIONINFO versInfo;
@@ -2656,6 +2658,9 @@ PBoolean PUDPSocket::ApplyQoS()
         versInfo.dwMinorVersion == 0)
       usesetsockopt = PTrue;         //Windows 2000 does not always support QOS_DESTADDR
   }
+#else
+  PBoolean usesetsockopt = PTrue;
+#endif
 
   PBoolean retval = PFalse;
   if (!usesetsockopt && sendAddress.IsValid() && sendPort != 0) {
@@ -2712,77 +2717,44 @@ PBoolean PUDPSocket::ApplyQoS()
 
 PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 {
-#ifdef _WIN32_WCE   //QOS not supported
-  return ConvertOSError(os_handle = os_socket(af, type, proto));
-#endif
-
 #if defined(_WIN32) && defined(P_QOS)
     
-  DWORD bufferSize = 0;
-  DWORD numProtocols, i;
-  LPWSAPROTOCOL_INFO installedProtocols, qosProtocol;
-
   //Try to find a QOS-enabled protocol
- 
-  PBoolean retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
-                                                            NULL,
-                                                            &bufferSize));
-    
-  if (numProtocols == SOCKET_ERROR && WSAGetLastError()!=WSAENOBUFS) 
-    return retval;
+  DWORD bufferSize = 0;
+  DWORD numProtocols = WSAEnumProtocols(proto != 0 ? &proto : NULL, NULL, &bufferSize);
+  if (!ConvertOSError(numProtocols) && WSAGetLastError() != WSAENOBUFS) 
+    return false;
 
-  installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
-  retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
-                                                            installedProtocols,
-                                                            &bufferSize));
-  if (numProtocols == SOCKET_ERROR) {
+  LPWSAPROTOCOL_INFO installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
+  numProtocols = WSAEnumProtocols(proto != 0 ? &proto : NULL, installedProtocols, &bufferSize);
+  if (!ConvertOSError(numProtocols)) {
     delete[] installedProtocols;
-    return retval;
+    return false;
   }
 
-  qosProtocol = installedProtocols;
-  PBoolean haveQoSproto = PFalse;
-
-  for (i=0; i<numProtocols; qosProtocol++, i++) {
+  LPWSAPROTOCOL_INFO qosProtocol = installedProtocols;
+  for (DWORD i = 0; i < numProtocols; qosProtocol++, i++) {
     if ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
         (qosProtocol->iSocketType == type) &&
         (qosProtocol->iAddressFamily == af)) {
-      haveQoSproto = PTrue;
+      os_handle = WSASocket(af, type, proto, qosProtocol, 0, WSA_FLAG_OVERLAPPED);
       break;
     }
   }
 
-  if (haveQoSproto) {
-    retval =  ConvertOSError(os_handle = WSASocket(af,
-                                                   type,
-                                                   proto,
-                                                   qosProtocol,
-                                                   0,
-                                                   WSA_FLAG_OVERLAPPED));
-  }
-  else
-  {    
-    retval = ConvertOSError(os_handle = WSASocket (af,
-                                                   type,
-                                                   proto,
-                                                   NULL,
-                                                   0,
-                                                   WSA_FLAG_OVERLAPPED));
-  }
-
   delete[] installedProtocols;
 
-  if (os_handle == INVALID_SOCKET)
-    return retval;
-#else
-  PBoolean retval = ConvertOSError(os_handle = os_socket(af, type, proto));
-#endif
+  if (!IsOpen())
+    os_handle = WSASocket(af, type, proto, NULL, 0, WSA_FLAG_OVERLAPPED);
 
-  return retval;
+  return ConvertOSError(os_handle);
+
+#else
+  return ConvertOSError(os_handle = os_socket(af, type, proto));
+#endif
 }
 
 #ifdef _WIN32
-#ifndef _WIN32_WCE
 #if P_QOS
 
 #define COULD_HAVE_QOS
@@ -2801,7 +2773,6 @@ static PBoolean CheckOSVersionFor(DWORD major, DWORD minor)
 }
 
 #endif // P_QOS
-#endif // _WIN32_WCE
 #endif // _WIN32
 
 PBoolean PUDPSocket::OpenSocket()
@@ -2873,6 +2844,16 @@ void PUDPSocket::GetLastReceiveAddress(Address & address, WORD & port)
   port    = lastReceivePort;
 }
 
+PBoolean PUDPSocket::IsAlternateAddress(const Address &, WORD)
+{
+  return false;
+}
+
+PBoolean PUDPSocket::DoPseudoRead(int & /*selectStatus*/)
+{
+   return false;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 PBoolean PICMPSocket::OpenSocket(int)
@@ -2889,16 +2870,16 @@ PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD port, char sep
 
   PINDEX pos = str.Find(separator);
   if (pos != P_MAX_INDEX) {
-    m_port    = (WORD)str.Mid(pos+1).AsInteger();
     if (!PIPSocket::GetHostAddress(str.Left(pos), m_address))
       return false;
+    m_port = (WORD)str.Mid(pos+1).AsInteger();
+  }
+  else {
+    if (!PIPSocket::GetHostAddress(str, m_address))
+      return false;
   }
-  else if (m_port == 0) 
-    return false;
-  else if (!PIPSocket::GetHostAddress(str, m_address))
-    return false;
 
-  return true;
+  return m_port != 0;
 }
 
 
