@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22083 $
+ * $Revision: 25062 $
  * $Author: rjongbloed $
- * $Date: 2009-02-17 11:21:49 +0000 (Tue, 17 Feb 2009) $
+ * $Date: 2011-01-13 22:14:36 -0600 (Thu, 13 Jan 2011) $
  */
 
 #include <ptlib.h>
@@ -50,22 +50,25 @@
 #define OID_GEN_CURRENT_PACKET_FILTER       0x0001010E
 #define OID_GEN_MEDIA_SUPPORTED             0x00010103
 
-#define NDIS_PACKET_TYPE_DIRECTED           0x0001
-#define NDIS_PACKET_TYPE_MULTICAST          0x0002
-#define NDIS_PACKET_TYPE_ALL_MULTICAST      0x0004
-#define NDIS_PACKET_TYPE_BROADCAST          0x0008
-#define NDIS_PACKET_TYPE_PROMISCUOUS        0x0020
+#if (WINVER <= 0x502)  // These are defined in Vista SDK
 
-typedef enum _NDIS_MEDIUM {
-    NdisMedium802_3,
-    NdisMedium802_5,
-    NdisMediumFddi,
-    NdisMediumWan,
-    NdisMediumLocalTalk,
-    NdisMediumDix,              // defined for convenience, not a real medium
-    NdisMediumArcnetRaw,
-    NdisMediumArcnet878_2
-} NDIS_MEDIUM, *PNDIS_MEDIUM;                    
+	#define NDIS_PACKET_TYPE_DIRECTED           0x0001
+	#define NDIS_PACKET_TYPE_MULTICAST          0x0002
+	#define NDIS_PACKET_TYPE_ALL_MULTICAST      0x0004
+	#define NDIS_PACKET_TYPE_BROADCAST          0x0008
+	#define NDIS_PACKET_TYPE_PROMISCUOUS        0x0020
+
+	typedef enum _NDIS_MEDIUM {
+		NdisMedium802_3,
+		NdisMedium802_5,
+		NdisMediumFddi,
+		NdisMediumWan,
+		NdisMediumLocalTalk,
+		NdisMediumDix,              // defined for convenience, not a real medium
+		NdisMediumArcnetRaw,
+		NdisMediumArcnet878_2
+	} NDIS_MEDIUM, *PNDIS_MEDIUM;  
+#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -1369,6 +1372,7 @@ public:
 };
 
 
+#if IPv6_ENABLED
 class PIPAdaptersAddressTable
 {
 public:
@@ -1399,7 +1403,6 @@ private:
 };
 
 
-#if IPv6_ENABLED
 #include <tchar.h>
 
 class PIPRouteTableIPv6 : public PIPRouteTable
@@ -1541,8 +1544,8 @@ PString PIPSocket::GetGatewayInterface(int version)
 
 PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(int version)
 {
-#if IPv6_ENABLED
   if (version == 6) {
+#if IPv6_ENABLED
     PIPRouteTableIPv6 routes;
     if (routes->NumEntries > 0) {
       PIPAdaptersAddressTable interfaces;
@@ -1565,8 +1568,10 @@ PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(int version)
         }
       }
     }
-  }
+#else
+    return GetDefaultIpAny();
 #endif
+  }
 
   PIPRouteTable routes;
   for (unsigned i = 0; i < routes->dwNumEntries; ++i) {
@@ -1614,57 +1619,78 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #pragma optimize("g", off)
 #endif
 
-bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
+class Win32RouteTableDetector : public PIPSocket::RouteTableDetector
 {
-  HANDLE handle = NULL;
-  OVERLAPPED overlap;
-  memset(&overlap, 0, sizeof(overlap));
+    PDynaLink  m_dll;
+    BOOL    (WINAPI * m_pCancelIPChangeNotify )(LPOVERLAPPED);
+    HANDLE     m_hCancel;
 
-  // Do this so compatible with older operating systems
-  PDynaLink dll("iphlpapi.dll");
-  BOOL (*pCancelIPChangeNotify)(LPOVERLAPPED) = NULL;
-  bool doNotify = dll.GetFunction("CancelIPChangeNotify", (PDynaLink::Function&)pCancelIPChangeNotify);
-
-  if (doNotify) {
-    DWORD error = NotifyAddrChange(&handle, &overlap);
-    if (error != ERROR_IO_PENDING) {
-      PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
-      doNotify = false;
+  public:
+    Win32RouteTableDetector()
+      : m_dll("iphlpapi.dll")
+      , m_hCancel(CreateEvent(NULL, TRUE, FALSE, NULL))
+    {
+      if (!m_dll.GetFunction("CancelIPChangeNotify", (PDynaLink::Function&)m_pCancelIPChangeNotify))
+        m_pCancelIPChangeNotify = NULL;
     }
-  }
 
-  if (!doNotify) {
-    if (cancellation != NULL)
-      return cancellation->Wait(timeout);
+    ~Win32RouteTableDetector()
+    {
+      if (m_hCancel != NULL)
+        CloseHandle(m_hCancel);
+    }
 
-    PThread::Sleep(timeout);
-    return false;
-  }
+    virtual bool Wait(const PTimeInterval & timeout)
+    {
+      HANDLE hNotify = NULL;
+      OVERLAPPED overlap;
 
-  if (cancellation == NULL)
-    return WaitForSingleObject(handle, timeout.GetInterval()) == WAIT_OBJECT_0;
+      if (m_pCancelIPChangeNotify != NULL) {
+        memset(&overlap, 0, sizeof(overlap));
+        DWORD error = NotifyAddrChange(&hNotify, &overlap);
+        if (error != ERROR_IO_PENDING) {
+          PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
+          hNotify = NULL;
+        }
+      }
 
-  HANDLE handles[2];
-  handles[0] = handle;
-  handles[1] = cancellation->GetHandle();
-  switch (WaitForMultipleObjects(2, handles, false, timeout.GetInterval())) {
-    case WAIT_OBJECT_0 :
-      return true;
+      if (hNotify == NULL)
+        return WaitForSingleObject(m_hCancel, timeout.GetInterval()) == WAIT_TIMEOUT;
 
-    case WAIT_OBJECT_0+1 :
-      pCancelIPChangeNotify(&overlap);
-      // Do next case
+      HANDLE handles[2];
+      handles[0] = hNotify;
+      handles[1] = m_hCancel;
+      switch (WaitForMultipleObjects(2, handles, false, INFINITE)) {
+        case WAIT_OBJECT_0 :
+          return true;
 
-    default :
-      return false;
-  }
-}
+        case WAIT_OBJECT_0+1 :
+          if (m_pCancelIPChangeNotify != NULL)
+            m_pCancelIPChangeNotify(&overlap);
+          // Do next case
+
+        default :
+          return false;
+      }
+    }
+
+    virtual void Cancel()
+    {
+      SetEvent(m_hCancel);
+    }
+};
 
 #ifdef _MSC_VER
 #pragma optimize("", on)
 #endif
 
 
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new Win32RouteTableDetector();
+}
+
+
 PIPSocket::Address PIPSocket::GetRouteAddress(PIPSocket::Address remoteAddress)
 {
   DWORD best;
