@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22318 $
+ * $Revision: 25324 $
  * $Author: rjongbloed $
- * $Date: 2009-03-29 18:50:35 -0500 (Sun, 29 Mar 2009) $
+ * $Date: 2011-03-15 00:51:31 -0500 (Tue, 15 Mar 2011) $
  */
 
 #pragma implementation "channel.h"
@@ -150,19 +150,32 @@ PBoolean PChannel::Read(void * buf, PINDEX len)
   if (os_handle < 0)
     return SetErrorValues(NotOpen, EBADF, LastReadError);
 
-  if (!PXSetIOBlock(PXReadBlock, readTimeout)) 
-    return PFalse;
+  while ((lastReadCount = ::read(os_handle, buf, len)) < 0) {
+    switch (errno) {
+      case EINTR :
+        break;
 
-  if (ConvertOSError(lastReadCount = ::read(os_handle, buf, len), LastReadError))
-    return lastReadCount > 0;
+      case EWOULDBLOCK :
+        if (readTimeout > 0) {
+          if (PXSetIOBlock(PXReadBlock, readTimeout))
+            break;
+          return false;
+        }
+        // Next case
 
-  lastReadCount = 0;
-  return PFalse;
+      default :
+        return ConvertOSError(-1);
+    }
+  }
+
+  return lastReadCount > 0;
 }
 
 
 PBoolean PChannel::Write(const void * buf, PINDEX len)
 {
+  lastWriteCount = 0;
+
   // if the os_handle isn't open, no can do
   if (os_handle < 0)
     return SetErrorValues(NotOpen, EBADF, LastWriteError);
@@ -172,17 +185,25 @@ PBoolean PChannel::Write(const void * buf, PINDEX len)
   flush();
   IOSTREAM_MUTEX_SIGNAL();
 
-  lastWriteCount = 0;
-  
   while (len > 0) {
 
     int result;
     while ((result = ::write(os_handle, ((char *)buf)+lastWriteCount, len)) < 0) {
-      if (errno != EWOULDBLOCK)
-        return ConvertOSError(-1, LastWriteError);
+      switch (errno) {
+        case EINTR :
+          break;
 
-      if (!PXSetIOBlock(PXWriteBlock, writeTimeout))
-        return PFalse;
+        case EWOULDBLOCK :
+          if (writeTimeout > 0) {
+            if (PXSetIOBlock(PXWriteBlock, writeTimeout))
+              break;
+            return false;
+          }
+          // Next case
+
+        default :
+          return ConvertOSError(-1, LastReadError);
+      }
     }
 
     lastWriteCount += result;
@@ -197,6 +218,11 @@ PBoolean PChannel::Write(const void * buf, PINDEX len)
   return ConvertOSError(0, LastWriteError);
 }
 
+PBoolean PChannel::Write(const void * buf, PINDEX len, const void * /*mark*/)
+{
+   return Write(buf,len);
+}
+
 #ifdef P_HAS_RECVMSG
 
 PBoolean PChannel::Read(const VectorOfSlice & slices)
@@ -347,6 +373,11 @@ PBoolean PChannel::ConvertOSError(int err, Errors & lastError, int & osError)
       lastError = Interrupted;
       break;
 
+    case EWOULDBLOCK :
+    case ETIMEDOUT :
+      lastError = Timeout;
+      break;
+
     case EEXIST:
       lastError = FileExists;
       break;
