@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23336 $
+ * $Revision: 25444 $
  * $Author: rjongbloed $
- * $Date: 2009-09-05 01:02:14 +0000 (Sat, 05 Sep 2009) $
+ * $Date: 2011-03-29 17:17:31 -0500 (Tue, 29 Mar 2011) $
  */
 
 #define _CONFIG_CXX
@@ -53,7 +53,7 @@
 #define environ (NULL)
 #endif
 
-#if defined(__NetBSD__)
+#if defined(P_NETBSD) || defined(P_OPENBSD)
 extern char **environ;
 #endif
 
@@ -128,6 +128,13 @@ class PXConfig : public PList<PXConfigSection>
     PBoolean      canSave;
 };
 
+
+static PBoolean IsComment(const PString& str)
+{
+  return str.GetLength() && strchr(";#", str[0]);
+}
+
+
 //
 // a dictionary of configurations, keyed by filename
 //
@@ -281,13 +288,24 @@ PBoolean PXConfig::WriteToFile(const PFilePath & filename)
 
   for (PINDEX i = 0; i < GetSize(); i++) {
     PXConfigSectionList & section = (*this)[i].GetList();
+
+    // If the line is a comment, output it as is
+    if (IsComment((*this)[i])) {
+      file << (*this)[i] << endl;
+      continue;
+    }
+
     file << "[" << (*this)[i] << "]" << endl;
     for (PINDEX j = 0; j < section.GetSize(); j++) {
       PXConfigValue & value = section[j];
       PStringArray lines = value.GetValue().Tokenise('\n', PTrue);
-      PINDEX k;
-      for (k = 0; k < lines.GetSize(); k++) 
-        file << value << "=" << lines[k] << endl;
+      // Preserve name/value pairs with no value, i.e. of the form "name="
+      if (lines.IsEmpty())
+	    file << value << "=" << endl;
+      else {
+        for (PINDEX k = 0; k < lines.GetSize(); k++) 
+          file << value << "=" << lines[k] << endl;
+      }
     }
     file << endl;
   }
@@ -330,11 +348,11 @@ PBoolean PXConfig::ReadFromFile(const PFilePath & filename)
     file >> line;
     line = line.Trim();
     if ((len = line.GetLength()) > 0) {
-
-      // ignore comments and blank lines 
-      char ch = line[0];
-      if ((len > 0) && (ch != ';') && (ch != '#')) {
-        if (ch == '[') {
+      // Preserve comments
+      if (IsComment(line))
+        Append(new PXConfigSection(line));
+      else {
+        if (line[0] == '[') {
           PCaselessString sectionName = (line.Mid(1,len-(line[len-1]==']'?2:1))).Trim();
           PINDEX  index;
           if ((index = GetValuesIndex(sectionName)) != P_MAX_INDEX)
@@ -376,6 +394,12 @@ void PXConfig::ReadFromEnvironment (char **envp)
   PXConfigSection * currentSection = new PXConfigSection("Options");
   Append(currentSection);
 
+  // can't save environment configs
+  canSave = PFalse;
+
+  if (envp == NULL)
+    return;
+
   while (*envp != NULL && **envp != '\0') {
     PString line(*envp);
     PINDEX equals = line.Find('=');
@@ -385,9 +409,6 @@ void PXConfig::ReadFromEnvironment (char **envp)
     }
     envp++;
   }
-
-  // can't save environment configs
-  canSave = PFalse;
 }
 
 PINDEX PXConfig::GetSectionsIndex(const PString & theSection) const
